library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity AESround is
    Port (  AESinput : in  STD_LOGIC_VECTOR (127 downto 0);
            AESoutput : out  STD_LOGIC_VECTOR (127 downto 0);
			AESkey : in  STD_LOGIC_VECTOR (127 downto 0));
end AESround;

architecture Behavioral of AESround is

	--components
	component ByteSub is
		port( 	BS_in :in std_logic_vector(7 downto 0 );
				BS_out :out std_logic_vector(7 downto 0 ));
	end component;
	
	component ShiftRow is
		port (	shiftrow_in : in std_logic_vector(127 downto 0);
				shiftrow_out : out std_logic_vector(127 downto 0));
	end component;

	component MixColumn is 
		port (	MC_in : in std_logic_vector (31 downto 0);
				MC_out : out std_logic_vector(31 downto 0));
	end component;
	
	component AddRoundKey is 
		port (	addroundkey_in : in std_logic_vector(127 downto 0);
				addroundkey_key : in std_logic_vector(127 downto 0);
				addroundkey_out : out std_logic_vector(127 downto 0));
	end component;

	
	--signals
	signal afterbs: std_logic_vector(127 downto 0);
	signal aftersr: std_logic_vector(127 downto 0);
	signal aftermc: std_logic_vector(127 downto 0);
	


begin

	gen_BS: for i in 15 downto 0 generate
	inst_BS: ByteSub
		port map(AESinput(8*i+7 downto 8*i), afterbs(8*i+7 downto 8*i));
	end generate;

	inst_SR: ShiftRow
		port map(afterbs(127 downto 0), aftersr(127 downto 0));

	gen_MC: for i in 3 downto 0 generate
		inst_MixColumn: MixColumn 
			port map(aftersr(32*i+31 downto 32*i), aftermc(32*i+31 downto 32*i));
	end generate;
	
	inst_ARK: AddRoundKey
		port map(aftermc(127 downto 0), AESkey(127 downto 0), AESoutput(127 downto 0));

	
end Behavioral;