library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity echo_round is
    Port (  round_in : in  STD_LOGIC_VECTOR (2047 downto 0);
            round_out : out  STD_LOGIC_VECTOR (2047 downto 0);
			round_counter : in  STD_LOGIC_VECTOR (2 downto 0);
			bit_counter:in std_logic_vector(63 downto 0);
			clk : in std_logic;
			start : in std_logic;
			rst : in std_logic;
			done : out std_logic);
end echo_round;

architecture Behavioral of echo_round is

	--component declarations
	component BigSubWords is 
	port (	subwords_in : in std_logic_vector(2047 downto 0);
			subwords_out : out std_logic_vector(2047 downto 0);
			counter : in std_logic_vector(63 downto 0);
			round_counter:in std_logic_vector(2 downto 0);
			clk : in std_logic;
			start : in std_logic;
			rst : in std_logic;
			done : out std_logic);
	end component;
	
	component BigShiftRow is 
	port (	shiftrow_in : in std_logic_vector(2047 downto 0);
			shiftrow_out : out std_logic_vector(2047 downto 0));
	end component;
	
	component BigMixColumn is 
	port (	mixcolumn_in : in std_logic_vector(2047 downto 0);
			mixcolumn_out : out std_logic_vector(2047 downto 0));
	end component;

	--signal declarations
	signal afterbsw: std_logic_vector(2047 downto 0);
	signal afterbsr: std_logic_vector(2047 downto 0);
	signal in_reg: std_logic_vector(2047 downto 0);
	
	signal startbsw: std_logic;
	signal donebsw: std_logic;

	--states of the echo-fsm
	type FSM_state is (S_IDLE, S_LOOP, S_DONE);
	signal state: FSM_state;

begin

	--componenten instances
	inst_BSW: BigSubWords
		port map(in_reg, afterbsw, bit_counter, round_counter,clk,startbsw,rst,donebsw);
				
	inst_BSR: BigShiftRow
		port map(afterbsw, afterbsr);

	inst_BMC: BigMixColumn
		port map(afterbsr, round_out);	
		
	--the fsm
	fsm: process(rst, clk)
	begin
		if rst = '0' then
			startbsw <= '0';
			done <= '0';
			in_reg<=in_reg;
			state <= S_IDLE;
			
		elsif clk'event and clk = '1' then
			case state is
				when S_IDLE =>
					if start = '1' then
						startbsw <= '1';
						done <= '0';
						in_reg<=round_in;
						state <= S_LOOP;
					else
						startbsw <= '0';
						done <= '0';
						in_reg<=in_reg;
						state <= S_IDLE;
					end if;
					
				when S_LOOP =>
					if donebsw= '1' then	
						startbsw <= '0';
						done <= '0';
						in_reg<=in_reg;
						state <= S_DONE;
					else
						startbsw <= '0';
						done <= '0';
						in_reg<=in_reg;
						state <= S_LOOP;
					end if;
				
				
				when S_DONE =>
					if start = '1' then
						startbsw <= '0';
						done <= '1';
						in_reg<=in_reg;
						state <= S_DONE;
					else
						startbsw <= '0';
						done <= '1';
						in_reg<=in_reg;
						state <= S_IDLE;
					end if;
					
				when others =>
					startbsw <= '0';
					done <= '0';
					in_reg<=in_reg;
					state <= S_IDLE;
					
			end case;
		end if;
	end process;
			
	
end Behavioral;

