library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity BigMixColumn is 
	port (	mixcolumn_in : in std_logic_vector(2047 downto 0);
			mixcolumn_out : out std_logic_vector(2047 downto 0);
			clk : in std_logic;
			start : in std_logic;
			rst : in std_logic;
			done : out std_logic);
end entity;

architecture BigMixColumn_arch of BigMixColumn is
	
	component MixColumn is 
	port (	MC_in : in std_logic_vector (31 downto 0);
			MC_out : out std_logic_vector(31 downto 0));
	end component;
	
	component reg32 is
	port(	reg_in: in STD_LOGIC_VECTOR(31 downto 0);
			clk: in STD_LOGIC;
			en: in STD_LOGIC;
			reg_out: out STD_LOGIC_VECTOR(31 downto 0));
	end component;
	
	signal AEScolumnsin: std_logic_vector(2047 downto 0);
	signal AEScolumnsout: std_logic_vector(2047 downto 0);
	
	signal sel: std_logic_vector(5 downto 0);
	signal sel_int: integer;
	signal muxout: std_logic_vector(31 downto 0);
	signal AESout: std_logic_vector(31 downto 0);
	
	signal AESregen: std_logic_vector(63 downto 0);
	
	--states of the echo-fsm
	type FSM_state is (S_IDLE, S_LOOP, S_DONE);
	signal state: FSM_state;

	
begin
	
	--Transform incoming data to a format that is easier to work with
	--create a big bitstring (2047) which directly contains 32bit words which can be connected to aesmixcolumn after eachother 
	--(=easier to play with the amount of aes blocks and therefor needed FSMcounter!)
	gen_MC1: for j in 3 downto 0 generate
		gen_MC2: for i in 15 downto 0 generate
			AEScolumnsin(j*512+i*32+31 downto j*512+i*32) <= mixcolumn_in(j*512+384+i*8+7 downto j*512+384+i*8) & mixcolumn_in(j*512+256+i*8+7 downto j*512+256+i*8) & mixcolumn_in(j*512+128+i*8+7 downto j*512+128+i*8) & mixcolumn_in(j*512+i*8+7 downto j*512+i*8);
		end generate;
	end generate;
	
	--instantce of AES MC
	inst_MC: MixColumn
		port map(muxout, AESout );
		
	--generate 64 128-bit registers
	gen_MC3: for i in 63 downto 0 generate
	inst_reg32: reg32
		port map (AESout,clk,AESregen(i),AEScolumnsout(i*32+31 downto i*32));
	end generate;
	
	sel_int <= CONV_INTEGER(sel);
	
	inputmux: process (sel,AEScolumnsin)
	begin  
		muxout <= AEScolumnsin(sel_int*32+31 downto sel_int*32);
	end process;
	
	
	--Transform working format to ECHO state
	gen_MC4: for j in 3 downto 0 generate
		gen_MC5: for i in 15 downto 0 generate
			mixcolumn_out(j*512+384+i*8+7 downto j*512+384+i*8) <= AEScolumnsout(j*512+i*32+31 downto j*512+i*32+24);
			mixcolumn_out(j*512+256+i*8+7 downto j*512+256+i*8) <= AEScolumnsout(j*512+i*32+23 downto j*512+i*32+16);
			mixcolumn_out(j*512+128+i*8+7 downto j*512+128+i*8) <= AEScolumnsout(j*512+i*32+15 downto j*512+i*32+8);
			mixcolumn_out(j*512+i*8+7 downto j*512+i*8) <= AEScolumnsout(j*512+i*32+7 downto j*512+i*32);
		end generate;
	end generate;
	
	
	
	--the fsm
	fsm: process(rst, clk)
	begin
		if rst = '0' then
			AESregen <= "0000000000000000000000000000000000000000000000000000000000000000";
			done <= '0';
			sel <= (others => '0');
			state <= S_IDLE;
			
		elsif clk'event and clk = '1' then
			case state is
				when S_IDLE =>
					if start = '1' then
						AESregen <= "0000000000000000000000000000000000000000000000000000000000000001";
						done <= '0';
						sel <= "000001";
						state <= S_LOOP;
					else
						AESregen <= "0000000000000000000000000000000000000000000000000000000000000000";
						done <= '0';
						sel <= (others => '0');
						state <= S_IDLE;
					end if;
					
				when S_LOOP =>
					if sel = "000000" then
						AESregen <= "0000000000000000000000000000000000000000000000000000000000000000";
						done <= '0';
						sel <= sel +  1 ;
						state <= S_DONE;
					else
						AESregen <= AESregen(62 downto 0) & AESregen(63);
						done <= '0';
						sel <= sel +  1 ;
						state <= S_LOOP;
					end if;
					
				when S_DONE =>
					if start = '1' then
						AESregen <= "0000000000000000000000000000000000000000000000000000000000000000";
						done <= '1';
						sel <= (others => '0');
						state <= S_DONE;
					else
						AESregen <= "0000000000000000000000000000000000000000000000000000000000000000";
						done <= '1';
						sel <= (others => '0');
						state <= S_IDLE;
					end if;
					
				when others =>
					AESregen <= "0000000000000000000000000000000000000000000000000000000000000000";
					done <= '0';
					sel <= (others => '0');
					state <= S_IDLE;
					
			end case;
		end if;
	end process;
	
	
	
end;
