library ieee;
use ieee.std_logic_1164.all;

entity BigMixColumn is 
	port (	mixcolumn_in : in std_logic_vector(2047 downto 0);
			mixcolumn_out : out std_logic_vector(2047 downto 0));
end entity;

architecture BigMixColumn_arch of BigMixColumn is
	
	component MixColumn is 
	port (	MC_in : in std_logic_vector (31 downto 0);
			MC_out : out std_logic_vector(31 downto 0));
	end component;
	
	signal AEScolumnsin: std_logic_vector(2047 downto 0);
	signal AEScolumnsout: std_logic_vector(2047 downto 0);
	
	
begin
	
	--Transform incoming data to a format that is easier to work with
	--create a big bitstring (2047) which directly contains 32bit words which can be connected to aesmixcolumn after eachother 
	--(=easier to play with the amount of aes blocks and therefor needed FSMcounter!)
	gen_MC1: for j in 3 downto 0 generate
		gen_MC2: for i in 15 downto 0 generate
			AEScolumnsin(j*512+i*32+31 downto j*512+i*32) <= mixcolumn_in(j*512+384+i*8+7 downto j*512+384+i*8) & mixcolumn_in(j*512+256+i*8+7 downto j*512+256+i*8) & mixcolumn_in(j*512+128+i*8+7 downto j*512+128+i*8) & mixcolumn_in(j*512+i*8+7 downto j*512+i*8);
		end generate;
	end generate;
	
	--generate 64 AES blocks (or less, but FSM needed then!)
	gen_MC3: for i in 63 downto 0 generate
	inst_MC: MixColumn
		port map(AEScolumnsin(i*32+31 downto i*32), AEScolumnsout(i*32+31 downto i*32) );
	end generate;
	
	
	--Transform working format to ECHO state
	gen_MC4: for j in 3 downto 0 generate
		gen_MC5: for i in 15 downto 0 generate
			mixcolumn_out(j*512+384+i*8+7 downto j*512+384+i*8) <= AEScolumnsout(j*512+i*32+31 downto j*512+i*32+24);
			mixcolumn_out(j*512+256+i*8+7 downto j*512+256+i*8) <= AEScolumnsout(j*512+i*32+23 downto j*512+i*32+16);
			mixcolumn_out(j*512+128+i*8+7 downto j*512+128+i*8) <= AEScolumnsout(j*512+i*32+15 downto j*512+i*32+8);
			mixcolumn_out(j*512+i*8+7 downto j*512+i*8) <= AEScolumnsout(j*512+i*32+7 downto j*512+i*32);
		end generate;
	end generate;
	
end;
