library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity BigSubWords is 
	port (	subwords_in : in std_logic_vector(2047 downto 0);
			subwords_out : out std_logic_vector(2047 downto 0);
			counter : in std_logic_vector(63 downto 0);
			round_counter:in std_logic_vector(2 downto 0);
			clk : in std_logic;
			start : in std_logic;
			rst : in std_logic;
			done : out std_logic);
end entity;

architecture BigSubWords_arch of BigSubWords is
	
	--components
	component AESround is
    Port (  AESinput : in  STD_LOGIC_VECTOR (127 downto 0);
            AESoutput : out  STD_LOGIC_VECTOR (127 downto 0);
			AESkey : in  STD_LOGIC_VECTOR (127 downto 0));
	end component;
	
	component reg512 is
	port(	reg_in: in STD_LOGIC_VECTOR(511 downto 0);
			clk: in STD_LOGIC;
			en: in STD_LOGIC;
			reg_out: out STD_LOGIC_VECTOR(511 downto 0));
	end component;
	
	
	--signals
	signal total_counter: std_logic_vector(2047 downto 0);
	signal total_counter_little_endian: std_logic_vector(2047 downto 0);
	signal total_counter_little_endian_part: std_logic_vector(511 downto 0);
	signal subwords_in_part: std_logic_vector(511 downto 0);
	signal state_after_aes1: std_logic_vector(511 downto 0);
	signal muxoutdata: std_logic_vector(511 downto 0);
	signal muxoutkey: std_logic_vector(511 downto 0);
	signal regout: std_logic_vector(511 downto 0);
	signal regouten: std_logic_vector(3 downto 0);
	signal sel : std_logic;
	signal selin : std_logic_vector(1 downto 0);
	signal counter_int: integer;
	signal round_counter_int: integer;
	
	
	--states of the echo-fsm
	type FSM_state is (S_IDLE, S_LOOP1, S_LOOP2, S_LOOP3, S_LOOP4, S_LOOP5, S_LOOP6, S_LOOP7, S_DONE);
	signal state: FSM_state;
	
begin
	
	----Dit is gewoon de versie waarbij voor ieder woord 2 aesrounds worden aangemaakt.
	
	-- optimalisatie: 1x4 stuks en 'muxed' terugkoppeling?
	
	round_counter_int <= CONV_INTEGER(round_counter);
	gen_BS0: for i in 15 downto 0 generate
		total_counter(128*i+127 downto 128*i+64)<= x"0000000000000000";--this is the small counter, addition of zero's
		total_counter(128*i+63 downto 128*i)<=  STD_LOGIC_VECTOR( counter+(15-i)+(round_counter_int*16));--this is the small counter, counter part
		total_counter_little_endian(128*i+127 downto 128*i)<=total_counter(128*i+7 downto 128*i+0) & total_counter(128*i+15 downto 128*i+8) & total_counter(128*i+23 downto 128*i+16) & total_counter(128*i+31 downto 128*i+24) & total_counter(128*i+39 downto 128*i+32) & total_counter(128*i+47 downto 128*i+40) & total_counter(128*i+55 downto 128*i+48) & total_counter(128*i+63 downto 128*i+56) & x"0000000000000000";
	end generate;
	
	gen_BS1: for i in 3 downto 0 generate
		inst_AESR1:AESround
		Port map(muxoutdata(128*i+127 downto 128*i), state_after_aes1(128*i+127 downto 128*i),muxoutkey(128*i+127 downto 128*i));
	end generate;
	
	--register to hold temp value
	inst_reg: reg512
		port map (state_after_aes1,clk,'1',regout);
	
	--registers to hold output
	gen_BSreg: for i in 3 downto 0 generate
	inst_reg: reg512
		port map (state_after_aes1,clk,regouten(i),subwords_out(i*512+511 downto i*512));
	end generate;
		
	
	
	
	mux_datainput: process (selin,subwords_in)
	begin  
		if selin = "00" then
			subwords_in_part <= subwords_in(2047 downto 1536);
		elsif selin = "01" then
			subwords_in_part<= subwords_in(1535 downto 1024);
		elsif selin = "10" then
			subwords_in_part<= subwords_in(1023 downto 512);
		else
			subwords_in_part<= subwords_in(511 downto 0);
		end if;
	end process;
	
	mux_keyinput: process (selin,total_counter_little_endian)
	begin  
		if selin = "00" then
			total_counter_little_endian_part <= total_counter_little_endian(2047 downto 1536);
		elsif selin = "01" then
			total_counter_little_endian_part <= total_counter_little_endian(1535 downto 1024);
		elsif selin = "10" then
			total_counter_little_endian_part <= total_counter_little_endian(1023 downto 512);
		else
			total_counter_little_endian_part <= total_counter_little_endian(511 downto 0);
		end if;
	end process;
	
	
	
	
	mux_data: process (sel,subwords_in_part,regout)
	begin  
		if sel = '0' then
			muxoutdata <= subwords_in_part;
		else
			muxoutdata <= regout;   --
		end if;
	end process;
	
	mux_key: process (sel,total_counter_little_endian_part)
	begin  
		if sel = '0' then
			muxoutkey <= total_counter_little_endian_part;
		else
			muxoutkey <= x"00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000";
		end if;
	end process;
	
	
	
	--the fsm
	fsm: process(rst, clk)
	begin
		if rst = '0' then
			done <= '0';
			sel <= '0';
			regouten <= "0000";
			selin <= "00";
			state <= S_IDLE;
			
		elsif clk'event and clk = '1' then
			case state is
				when S_IDLE =>
					if start = '1' then
						done <= '0';
						sel <= '0';
						regouten <= "0000";
						selin <= "00";
						state <= S_LOOP1;
					else
						done <= '0';
						sel <= '0';
						regouten <= "0000";
						selin <= "00";
						state <= S_IDLE;
					end if;
					
				when S_LOOP1 =>
						done <= '0';
						sel <= '1' ;
						regouten <= "1000";
						selin <= "01";
						state <= S_LOOP2;
						
				when S_LOOP2 =>
						done <= '0';
						sel <= '0' ;
						regouten <= "0000";
						selin <= "01";
						state <= S_LOOP3;
						
				when S_LOOP3 =>
						done <= '0';
						sel <= '1' ;
						regouten <= "0100";
						selin <= "10";
						state <= S_LOOP4;
				
				when S_LOOP4 =>
						done <= '0';
						sel <= '0' ;
						regouten <= "0000";
						selin <= "10";
						state <= S_LOOP5;
				
				when S_LOOP5 =>
						done <= '0';
						sel <= '1' ;
						regouten <= "0010";
						selin <= "11";
						state <= S_LOOP6;
				
				when S_LOOP6 =>
						done <= '0';
						sel <= '0' ;
						regouten <= "0000";
						selin <= "11";
						state <= S_LOOP7;
				
				when S_LOOP7 =>
						done <= '0';
						sel <= '1' ;
						regouten <= "0001";
						selin <= "01";
						state <= S_DONE;
					
				when S_DONE =>
					if start = '1' then
						done <= '1';
						sel <= '0';
						regouten <= "0000";
						selin <= "00";
						state <= S_DONE;
					else
						done <= '1';
						sel <= '0';
						regouten <= "0000";
						selin <= "00";
						state <= S_IDLE;
					end if;
					
				when others =>
					done <= '0';
					sel <= '0';
					regouten <= "0000";
					selin <= "00";
					state <= S_IDLE;
					
			end case;
		end if;
	end process;
	
end;
