library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity BigSubWords is 
	port (	subwords_in : in std_logic_vector(2047 downto 0);
			subwords_out : out std_logic_vector(2047 downto 0);
			counter : in std_logic_vector(63 downto 0);
			round_counter:in std_logic_vector(2 downto 0);
			clk : in std_logic;
			start : in std_logic;
			rst : in std_logic;
			done : out std_logic);
end entity;

architecture BigSubWords_arch of BigSubWords is
	
	--components
	component AESround is
    Port (  AESinput : in  STD_LOGIC_VECTOR (127 downto 0);
            AESoutput : out  STD_LOGIC_VECTOR (127 downto 0);
			AESkey : in  STD_LOGIC_VECTOR (127 downto 0));
	end component;
	
	component reg256 is
	port(	reg_in: in STD_LOGIC_VECTOR(255 downto 0);
			clk: in STD_LOGIC;
			en: in STD_LOGIC;
			reg_out: out STD_LOGIC_VECTOR(255 downto 0));
	end component;
	
	
	--signals
	signal total_counter: std_logic_vector(2047 downto 0);
	signal total_counter_little_endian: std_logic_vector(2047 downto 0);
	
	signal state_after_mux: std_logic_vector(255 downto 0);
	signal state_after_aes1: std_logic_vector(255 downto 0);
	signal state_after_reg: std_logic_vector(255 downto 0);
	signal state_after_aes2: std_logic_vector(255 downto 0);
	
	signal zeros: std_logic_vector(127 downto 0);
	
	signal muxoutkey: std_logic_vector(255 downto 0);
	signal tot_out: std_logic_vector(2047 downto 0);
	signal regen : std_logic_vector(7 downto 0);
	signal sel : std_logic_vector(2 downto 0);
	signal round_counter_int: integer;
	
	--states of the echo-fsm
	type FSM_state is (S_IDLE, S_LOOP, S_DONE);
	signal state: FSM_state;
	
begin
	
	----Dit is gewoon de versie waarbij voor ieder woord 2 aesrounds worden aangemaakt.
	
	-- optimalisatie: 2x2 stuks en pipelined
	
	zeros <= x"00000000000000000000000000000000";
	
	round_counter_int <= CONV_INTEGER(round_counter);
	gen_BS0: for i in 15 downto 0 generate
		total_counter(128*i+127 downto 128*i+64)<= x"0000000000000000";--this is the small counter, addition of zero's
		total_counter(128*i+63 downto 128*i)<=  STD_LOGIC_VECTOR( counter+(15-i)+(round_counter_int*16));--this is the small counter, counter part
		total_counter_little_endian(128*i+127 downto 128*i)<=total_counter(128*i+7 downto 128*i+0) & total_counter(128*i+15 downto 128*i+8) & total_counter(128*i+23 downto 128*i+16) & total_counter(128*i+31 downto 128*i+24) & total_counter(128*i+39 downto 128*i+32) & total_counter(128*i+47 downto 128*i+40) & total_counter(128*i+55 downto 128*i+48) & total_counter(128*i+63 downto 128*i+56) & x"0000000000000000";
	end generate;
	
	gen_BS1: for i in 1 downto 0 generate
		inst_AESR1:AESround
		Port map(state_after_mux(128*i+127 downto 128*i), state_after_aes1(128*i+127 downto 128*i),muxoutkey(128*i+127 downto 128*i));
		inst_AESR2:AESround
		Port map(state_after_reg(128*i+127 downto 128*i), state_after_aes2(128*i+127 downto 128*i),zeros);
	end generate;
	
	inst_reg_pipeline: reg256
		port map (state_after_aes1,clk,'1',state_after_reg);
	
	gen_BS2: for i in 7 downto 0 generate
	inst_reg: reg256
		port map (state_after_aes2,clk,regen(i),tot_out(i*256+255 downto i*256));
	end generate;
	
	subwords_out <= tot_out;
		
	
	
	--mux om input te kiezen
	mux_input: process (sel,subwords_in)
	begin  
		if sel = "000" then
			state_after_mux <= subwords_in(255 downto 0);
		elsif sel = "001" then
			state_after_mux <= subwords_in(511 downto 256);
		elsif sel = "010" then
			state_after_mux <= subwords_in(767 downto 512);
		elsif sel = "011" then
			state_after_mux <= subwords_in(1023 downto 768);
		elsif sel = "100" then
			state_after_mux <= subwords_in(1279 downto 1024);
		elsif sel = "101" then
			state_after_mux <= subwords_in(1535 downto 1280);
		elsif sel = "110" then
			state_after_mux <= subwords_in(1791 downto 1536);
		else
			state_after_mux <= subwords_in(2047 downto 1792);
		end if;
	end process;
	
	--mux om key1 te kiezen
	mux_key: process (sel,total_counter_little_endian)
	begin  
		if sel = "000" then
			muxoutkey <= total_counter_little_endian(255 downto 0);
		elsif sel = "001" then
			muxoutkey <= total_counter_little_endian(511 downto 256);
		elsif sel = "010" then
			muxoutkey <= total_counter_little_endian(767 downto 512);
		elsif sel = "011" then
			muxoutkey <= total_counter_little_endian(1023 downto 768);
		elsif sel = "100" then
			muxoutkey <= total_counter_little_endian(1279 downto 1024);
		elsif sel = "101" then
			muxoutkey <= total_counter_little_endian(1535 downto 1280);
		elsif sel = "110" then
			muxoutkey <= total_counter_little_endian(1791 downto 1536);
		else
			muxoutkey <= total_counter_little_endian(2047 downto 1792); 
		end if;
	end process;
	
	
	
	--the fsm
	fsm: process(rst, clk)
	begin
		if rst = '0' then
			regen <= "00000000";
			done <= '0';
			sel <= "000";
			state <= S_IDLE;
			
		elsif clk'event and clk = '1' then
			case state is
				when S_IDLE =>
					if start = '1' then
						regen <= "10000000";
						done <= '0';
						sel <= "000";
						state <= S_LOOP;
					else
						regen <= "00000000";
						done <= '0';
						sel <= "000";
						state <= S_IDLE;
					end if;
					
				when S_LOOP =>
					if sel= "111" then	
						regen <= regen(6 downto 0) & regen(7);
						done <= '0';
						sel <= sel+1 ;
						state <= S_DONE;
					else
						regen <= regen(6 downto 0) & regen(7);
						done <= '0';
						sel <= sel+1 ;
						state <= S_LOOP;
					end if;
				
				
				when S_DONE =>
					if start = '1' then
						regen <= "00000000";
						done <= '1';
						sel <= "000";
						state <= S_DONE;
					else
						regen <= "00000000";
						done <= '1';
						sel <= "000";
						state <= S_IDLE;
					end if;
					
				when others =>
					regen <= "00000000";
					done <= '0';
					sel <= "000";
					state <= S_IDLE;
					
			end case;
		end if;
	end process;
	
end;
