library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity BigSubWords is 
	port (	subwords_in : in std_logic_vector(2047 downto 0);
			subwords_out : out std_logic_vector(2047 downto 0);
			counter : in std_logic_vector(63 downto 0);
			round_counter:in std_logic_vector(2 downto 0);
			clk : in std_logic;
			start : in std_logic;
			rst : in std_logic;
			done : out std_logic);
end entity;

architecture BigSubWords_arch of BigSubWords is
	
	--components
	component AESround is
    Port (  AESinput : in  STD_LOGIC_VECTOR (127 downto 0);
            AESoutput : out  STD_LOGIC_VECTOR (127 downto 0);
			AESkey : in  STD_LOGIC_VECTOR (127 downto 0));
	end component;
	
	component reg1024 is
	port(	reg_in: in STD_LOGIC_VECTOR(1023 downto 0);
			clk: in STD_LOGIC;
			en: in STD_LOGIC;
			reg_out: out STD_LOGIC_VECTOR(1023 downto 0));
	end component;
	
	
	--signals
	signal total_counter: std_logic_vector(2047 downto 0);
	signal total_counter_little_endian: std_logic_vector(2047 downto 0);
	
	signal state_after_mux: std_logic_vector(1023 downto 0);
	signal state_after_aes1: std_logic_vector(1023 downto 0);
	signal state_after_reg: std_logic_vector(1023 downto 0);
	signal state_after_aes2: std_logic_vector(1023 downto 0);
	
	signal zeros: std_logic_vector(127 downto 0);
	
	signal muxoutkey: std_logic_vector(1023 downto 0);
	signal tot_out: std_logic_vector(2047 downto 0);
	signal regen : std_logic_vector(1 downto 0);
	signal sel : std_logic;
	signal round_counter_int: integer;
	
	--states of the echo-fsm
	type FSM_state is (S_IDLE, S_INPUT1, S_INPUT2, S_DONE);
	signal state: FSM_state;
	
begin
	
	----Dit is gewoon de versie waarbij voor ieder woord 2 aesrounds worden aangemaakt.
	
	-- optimalisatie: 2x8 stuks en pipelined
	
	zeros <= x"00000000000000000000000000000000";
	
	round_counter_int <= CONV_INTEGER(round_counter);
	gen_BS0: for i in 15 downto 0 generate
		total_counter(128*i+127 downto 128*i+64)<= x"0000000000000000";--this is the small counter, addition of zero's
		total_counter(128*i+63 downto 128*i)<=  STD_LOGIC_VECTOR( counter+(15-i)+(round_counter_int*16));--this is the small counter, counter part
		total_counter_little_endian(128*i+127 downto 128*i)<=total_counter(128*i+7 downto 128*i+0) & total_counter(128*i+15 downto 128*i+8) & total_counter(128*i+23 downto 128*i+16) & total_counter(128*i+31 downto 128*i+24) & total_counter(128*i+39 downto 128*i+32) & total_counter(128*i+47 downto 128*i+40) & total_counter(128*i+55 downto 128*i+48) & total_counter(128*i+63 downto 128*i+56) & x"0000000000000000";
	end generate;
	
	gen_BS1: for i in 7 downto 0 generate
		inst_AESR1:AESround
		Port map(state_after_mux(128*i+127 downto 128*i), state_after_aes1(128*i+127 downto 128*i),muxoutkey(128*i+127 downto 128*i));
		inst_AESR2:AESround
		Port map(state_after_reg(128*i+127 downto 128*i), state_after_aes2(128*i+127 downto 128*i),zeros);
	end generate;
	
	inst_reg_pipeline: reg1024
		port map (state_after_aes1,clk,'1',state_after_reg);
	
	gen_BS2: for i in 1 downto 0 generate
	inst_reg: reg1024
		port map (state_after_aes2,clk,regen(i),tot_out(i*1024+1023 downto i*1024));
	end generate;
	
	subwords_out <= tot_out;
		
	
	
	--mux om input te kiezen (eerste of laatste helft)
	mux_input: process (sel,subwords_in)
	begin  
		if sel = '0' then
			state_after_mux <= subwords_in(1023 downto 0);
		else
			state_after_mux <= subwords_in(2047 downto 1024);
		end if;
	end process;
	
	--mux om key1 te kiezen (eerste of laatste helft)
	mux_key: process (sel,total_counter_little_endian)
	begin  
		if sel = '0' then
			muxoutkey <= total_counter_little_endian(1023 downto 0);
		else
			muxoutkey <= total_counter_little_endian(2047 downto 1024); 
		end if;
	end process;
	
	
	
	--the fsm
	fsm: process(rst, clk)
	begin
		if rst = '0' then
			regen <= "00";
			done <= '0';
			sel <= '0';
			state <= S_IDLE;
			
		elsif clk'event and clk = '1' then
			case state is
				when S_IDLE =>
					if start = '1' then
						regen <= "00";
						done <= '0';
						sel <= '0';
						state <= S_INPUT1;
					else
						regen <= "00";
						done <= '0';
						sel <= '0';
						state <= S_IDLE;
					end if;
					
				when S_INPUT1 =>
						regen <= "01";
						done <= '0';
						sel <= '1' ;
						state <= S_INPUT2;
						
				when S_INPUT2 =>
						regen <= "10";
						done <= '0';
						sel <= '1' ;
						state <= S_DONE;
					
				when S_DONE =>
					if start = '1' then
						regen <= "00";
						done <= '1';
						sel <= '0';
						state <= S_DONE;
					else
						regen <= "00";
						done <= '1';
						sel <= '0';
						state <= S_IDLE;
					end if;
					
				when others =>
					regen <= "00";
					done <= '0';
					sel <= '0';
					state <= S_IDLE;
					
			end case;
		end if;
	end process;
	
end;
