library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity echo_full is port(
	clk, rst, init, add, final: in std_logic;
	done: out std_logic;
	data_in: in std_logic_vector(1535 downto 0);
	data_bits: in std_logic_vector(10 downto 0);
	hash_out: out std_logic_vector(255 downto 0));
end echo_full;

architecture Behavioral of echo_full is

	component echo_block is
    Port (  chain_in : in  STD_LOGIC_VECTOR (511 downto 0);
            message_block : in  STD_LOGIC_VECTOR (1535 downto 0);
            rst, clk : in STD_LOGIC;
			start : in  STD_LOGIC;
            done : out  STD_LOGIC;
			bit_counter : in STD_LOGIC_VECTOR(63 downto 0);
			chain_outout: out STD_LOGIC_VECTOR(511 downto 0));
	end component;

	type FSMstate is (S_IDLE, S_INIT, S_ADD, S_PRE_FINAL, S_ALMOST_FINAL, S_FINAL);
	signal next_state: FSMstate := S_IDLE;
	
	signal IV : std_logic_vector(511 downto 0);
	signal hashval : std_logic_vector(511 downto 0);
	signal chain_mux : std_logic_vector(511 downto 0);
	
	signal last_data : std_logic_vector(143 downto 0);
	signal sel_chain : std_logic;
	
	signal sel_data1 : std_logic;
	signal sel_data2 : std_logic;
	signal sel_counter : std_logic;
	
	signal echo_start : std_logic;
	signal echo_done : std_logic;
	signal echo_rst : std_logic;
	signal chaining_reg : std_logic_vector(511 downto 0);
	signal data_reg : std_logic_vector(1535 downto 0);
	signal bitcounter : std_logic_vector(63 downto 0);
	signal bitcounter_mux : std_logic_vector(63 downto 0);
	
	signal data_mux : std_logic_vector(1535 downto 0);
	signal data_mux1 : std_logic_vector(1535 downto 0);
	signal data_mux2 : std_logic_vector(143 downto 0);

begin

	echo_function: echo_block port map(chaining_reg, data_reg, rst, clk, echo_start, echo_done, bitcounter_mux, hashval);

	IV <= x"00010000000000000000000000000000000100000000000000000000000000000001000000000000000000000000000000010000000000000000000000000000"; --IV for 256 bit hash (128 bits x 4 =512 bits)
	last_data(143 downto 0) <= ( x"0001" & bitcounter(7 downto 0)  & bitcounter(15 downto 8) & bitcounter(23 downto 16) & bitcounter(31 downto 24) & bitcounter(39 downto 32) & bitcounter(47 downto 40) & bitcounter(55 downto 48) & bitcounter(63 downto 56) & x"0000000000000000" );  --remember that we're using little endian!! so we need to convert the bytes of our counter
	
	
	hash_out <= hashval(511 downto 256);


	mux_bitcounter: process(sel_counter, bitcounter)
	begin
		if sel_counter='1' then	
			bitcounter_mux <= (others =>'0');
		else
			bitcounter_mux <= bitcounter;
		end if;
	end process;
	
	mux_chain: process(sel_chain, IV, hashval)
	begin
		if sel_chain='1' then	
			chain_mux <= hashval;
		else
			chain_mux <= IV;
		end if;
	end process;
	
	mux_data1: process(sel_data1, data_in)
	begin
		if sel_data1='1' then	
			data_mux1 <= (others=>'0');
		else
			data_mux1 <= data_in;
		end if;
	end process;
	
	mux_data2: process(sel_data2, last_data, data_mux1)
	begin
		if sel_data2='1' then	
			data_mux2 <= last_data;
		else
			data_mux2 <= data_mux1(143 downto 0);
		end if;
	end process;
	data_mux <= data_mux1(1535 downto 144) & data_mux2;

	FSM_FULL: process(rst,clk)

	begin

		if rst = '0' then --laag actieve reset
			echo_rst <= '0';
			echo_start <= '0';
			sel_chain <= '0';
			sel_data1 <= '0';
			sel_data2 <= '0';
			sel_counter<='0';
			chaining_reg <= chaining_reg;
			data_reg <= data_mux;
			done <= '0';
			bitcounter <= (others => '0');
			next_state <= S_IDLE;

		elsif clk'event and clk = '1' then
			case next_state is
				when S_IDLE =>
					if init = '1' and add = '0' and final = '0' then 
						echo_rst <= '1';
						echo_start <= '0';
						sel_chain <= '0';
						sel_data1 <= '0';
						sel_data2 <= '0';
						sel_counter<='0';
						chaining_reg <= chain_mux;
						data_reg <= data_reg;
						done <= '0';
						bitcounter <= (others => '0');
						next_state <= S_INIT;
						
					elsif init = '0' and add = '1' and final = '0' then 
						echo_rst <= '1';
						echo_start <= '0';
						sel_chain <= sel_chain;
						sel_data1 <= '0';
						sel_data2 <= '0';
						sel_counter<='0';
						chaining_reg <= chain_mux;
						data_reg <= data_mux;
						done <= '0';
						bitcounter <= bitcounter + data_bits;
						next_state <= S_ADD;
						
					elsif init = '0' and add = '0' and final = '1' then 
						if data_bits < 1391 then 
							echo_rst <= '1';
							echo_start <= '0';
							sel_chain <= '1';
							sel_data1 <= '0';
							sel_data2 <= '1';
							sel_counter<='0';
							chaining_reg <= chain_mux;
							data_reg <= data_reg;
							done <= '0';
							bitcounter <= bitcounter + data_bits;
							next_state <= S_FINAL;
					
						else
							echo_rst <= '1';
							echo_start <= '0';
							sel_chain <= '1';
							sel_data1 <= '0';
							sel_data2 <= '0';
							sel_counter<='0';
							chaining_reg <= chain_mux;
							data_reg <= data_reg;
							done <= '0';
							bitcounter <= bitcounter + data_bits;
							next_state <= S_PRE_FINAL;
						end if;
						
						
					else
						echo_rst <= '1';
						echo_start <= '0';
						sel_chain <= sel_chain;
						sel_data1 <= '0';
						sel_data2 <= '0';
						sel_counter<='0';
						chaining_reg <= chaining_reg;
						data_reg <= data_mux;
						done <= '0';
						bitcounter <= bitcounter;
						next_state <= S_IDLE;
						
					end if;
				
				when S_INIT =>
						echo_rst <= '1';
						echo_start <= '0';
						sel_chain <= '0';
						sel_data1 <= '0';
						sel_data2 <= '0';
						sel_counter<='0';
						chaining_reg <= chaining_reg;
						data_reg <= data_mux;
						done <= '1';
						bitcounter <= (others => '0');
						next_state <= S_IDLE;

				
				when S_ADD =>
					if echo_done = '1' then 
						echo_rst <= '1';
						echo_start <= '0';
						sel_chain <= '1';
						sel_data1 <= '0';
						sel_data2 <= '0';
						sel_counter<='0';
						chaining_reg <= chaining_reg;
						data_reg <= data_mux;
						done <= '1';
						bitcounter <= bitcounter;
						next_state <= S_IDLE;
					
					else
						echo_rst <= '1';
						echo_start <= '1';
						sel_chain <= sel_chain;
						sel_data1 <= '0';
						sel_data2 <= '0';
						sel_counter<='0';
						chaining_reg <= chaining_reg;
						data_reg <= data_mux;
						done <= '0';
						bitcounter <= bitcounter;
						next_state <= S_ADD;
					end if;
							
				when S_PRE_FINAL =>
						if echo_done = '1' then 
							echo_rst <= '1';
							echo_start <= '0';
							sel_chain <= '1';
							sel_data1 <= '1';
							sel_data2 <= '1';
							sel_counter<='0';
							chaining_reg <= chain_mux;
							data_reg <= data_mux;
							done <= '0';
							bitcounter <= bitcounter;
							next_state <= S_ALMOST_FINAL;
						
						else
							echo_rst <= '1';
							echo_start <= '1';
							sel_chain <= '1';
							sel_data1 <= sel_data1;
							sel_data2 <= sel_data2;
							sel_counter<='0';
							chaining_reg <= chaining_reg;
							data_reg <= data_mux;
							done <= '0';
							bitcounter <= bitcounter;
							next_state <= S_PRE_FINAL;
						
						end if;	
						
				when S_ALMOST_FINAL =>
							echo_rst <= '1';
							echo_start <= '0';
							sel_chain <= '1';
							sel_data1 <= '1';
							sel_data2 <= '1';
							sel_counter<='1';
							chaining_reg <= chaining_reg;
							data_reg <= data_mux;
							done <= '0';
							bitcounter <= bitcounter;
						if echo_done = '1' then 
							next_state <= S_ALMOST_FINAL;
						else
							next_state <= S_FINAL;
						end if;
					
				when S_FINAL =>
						if echo_done = '1' then 
							echo_rst <= '1';
							echo_start <= '0';
							sel_chain <= '1';
							sel_data1 <= '0';
							sel_data2 <= '0';
							sel_counter<='0';
							chaining_reg <= chaining_reg;
							data_reg <= data_reg;
							done <= '1';
							bitcounter <= bitcounter;
							next_state <= S_IDLE;
						
						else
							echo_rst <= '1';
							echo_start <= '1';
							sel_chain <= '1';
							sel_data1 <= sel_data1;
							sel_data2 <= sel_data2;
							sel_counter<=sel_counter;
							chaining_reg <= chaining_reg;
							data_reg <= data_mux;
							done <= '0';
							bitcounter <= bitcounter;
							next_state <= S_FINAL;
						
						end if;
						
				when others =>
					echo_rst <= '0';
					echo_start <= '0';
					sel_chain <= '0';
					sel_data1 <= '0';
					sel_data2 <= '0';
					sel_counter<='0';
					chaining_reg <= chaining_reg;
					data_reg <= data_mux;
					done <= '0';
					bitcounter <= (others => '0');
					next_state <= S_IDLE;
			
			end case;
		end if;
	end process;



end Behavioral;

