
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity echo_full_buffer is port(
	clk, rst, init, add, final: in std_logic;
	done: out std_logic;
	data: in std_logic;
	hashval: out std_logic_vector(255 downto 0));
end echo_full_buffer;

architecture Behavioral of echo_full_buffer is

	component echo_full is port(
			clk, rst, init, add, final: in std_logic;
			done: out std_logic;
			data_in: in std_logic_vector(1535 downto 0);
			data_bits: in std_logic_vector(10 downto 0);
			hash_out: out std_logic_vector(255 downto 0));
	end component;
	
	component shiftreg1536 is port(	
			shiftin: in STD_LOGIC_VECTOR(7 downto 0);
			clk: in STD_LOGIC;
			rst: in STD_LOGIC;
			parallel_out: out STD_LOGIC_VECTOR(1535 downto 0);
			bits: in STD_LOGIC;
			enable: in STD_LOGIC);
	end component;

	type FSMstate is (S_IDLE, S_INIT, S_RESET, S_ADD1_FINAL, S_PRE_FINAL, S_FINAL, S_DONE);
	signal next_state: FSMstate := S_IDLE;
	
	signal ef_init: std_logic;
	signal ef_add: std_logic;
	signal ef_final: std_logic;
	signal ef_done: std_logic;
	
	signal shift_parallel_out: std_logic_vector(1535 downto 0);
	signal datareg: std_logic_vector(1535 downto 0);
	signal counter: std_logic_vector(10 downto 0);
	signal counterreg: std_logic_vector(10 downto 0);
	--signal paddingcounter: std_logic_vector(10 downto 0);
	signal shiftregenable: std_logic;
	signal padding: std_logic;
	signal select_padding_bit: std_logic;
	signal enable_add: std_logic;
	signal mux_input_out: std_logic;
	
	signal shiftinfix: std_logic_vector(7 downto 0);

begin
	shiftinfix <= "0000000" & mux_input_out;
	
	inst_echo_full: echo_full port map(clk, rst, ef_init, ef_add, ef_final, ef_done, datareg, counterreg, hashval);
	inst_shiftreg1536: shiftreg1536 port map(shiftinfix, clk, rst,shift_parallel_out, '0', shiftregenable);
	
	shiftregenable <= (add and enable_add) or padding;
	
	mux_input_select: process(padding, data, select_padding_bit)
	begin
		if padding = '0' then
			mux_input_out <= data;
		else
			mux_input_out <= select_padding_bit;
		end if;
	end process;
	


	FSM_FULL_BUFFER: process(rst,clk)

	begin

		if rst = '0' then --laag actieve reset
			counter <= (others => '0');
			ef_init <= '0';
			ef_add <= '0';
			ef_final <= '0';
			datareg <= datareg;
			counterreg <= counterreg;
			enable_add <= '1';
			padding <= '0';
			select_padding_bit <= '1';
			done <= '0';
			next_state <= S_IDLE;

		elsif clk'event and clk = '1' then
			case next_state is
				when S_IDLE =>
					if init = '1' and add = '0' and final = '0' then 
						counter <= (others => '0');
						ef_init <= '1';
						ef_add <= '0';
						ef_final <= '0';
						datareg <= datareg;
						counterreg <= counterreg;
						enable_add <= '0';
						padding <= '0';
						select_padding_bit <= '1';
						done <= '0';
						next_state <= S_INIT;
							
					elsif init = '0' and add = '0' and final = '1' then 
						counter <= counter+1;  --???? 1 weg? zo wast eerst!
						ef_init <= '0';
						ef_add <= '0';
						ef_final <= '0';
						datareg <= datareg;
						counterreg <= counterreg;
						enable_add <= '0';
						padding <= '1';
						select_padding_bit <= '1';
						done <= '0';
						next_state <= S_ADD1_FINAL;
						
					else
						if counter = "11000000000" then
							ef_init <= '0';
							ef_add <= '1';
							ef_final <= '0';
							datareg <= shift_parallel_out;
							counterreg <= counter;
							enable_add <= '1';
							padding <= '0';
							select_padding_bit <= '1';
							done <= '0';
							next_state <= S_IDLE;
							
							if add = '1' then
								counter <= "00000000001";
							else
								counter <= (others => '0');
							end if;
						
						
						elsif add = '1' then
							counter <= counter+1;
							ef_init <= '0';
							ef_add <= '0';
							ef_final <= '0';
							datareg <= datareg;
							counterreg <= counterreg;
							enable_add <= '1';
							padding <= '0';
							select_padding_bit <= '1';
							done <= '0';
							next_state <= S_IDLE;
							
						else
							counter <= counter;
							ef_init <= '0';
							ef_add <= '0';
							ef_final <= '0';
							datareg <= datareg;
							counterreg <= counterreg;
							enable_add <= '1';
							padding <= '0';
							select_padding_bit <= '1';
							done <= '0';
							next_state <= S_IDLE;
						end if;
					end if;
				
				
				when S_INIT =>
				
					counter <= counter;
					ef_init <= '0';
					ef_add <= '0';
					ef_final <= '0';
					datareg <= datareg;
					counterreg <= counterreg;
					padding <= '0';
					select_padding_bit <= '1';
					
					
					if ef_done = '1' then
						enable_add <= '1';
						next_state <= S_IDLE;
						done <= '1';
					else 
						enable_add <= '0';
						next_state <= S_INIT;
						done <= '0';
					end if;
	
	
				when S_ADD1_FINAL =>
					counter <= counter+1;
					ef_init <= '0';
					ef_add <= '0';
					ef_final <= '0';
					datareg <= datareg;
					counterreg <= counter;
					enable_add <= '0';
					padding <= '1';
					select_padding_bit <= '0';
					done <= '0';
					next_state <= S_FINAL;
				
				
				when S_FINAL =>

					if counter = "11000000000" then
						counter <= (others=>'0');
						ef_init <= '0';
						ef_add <= '0';
						ef_final <= '1';
						datareg <= shift_parallel_out;
						counterreg <= counterreg;
						enable_add <= '0';
						padding <= '0';
						select_padding_bit <= '0';
						done <= '0';
						next_state <= S_DONE;
					else 
						counter <= counter+1;
						ef_init <= '0';
						ef_add <= '0';
						ef_final <= '0';
						datareg <= datareg;
						counterreg <= counterreg;
						enable_add <= '0';
						padding <= '1';
						select_padding_bit <= '0';
						done <= '0';
						next_state <= S_FINAL;
						
					end if;

					
					
				when S_DONE =>
					if ef_done = '1' then
						counter <= (others=>'0');
						ef_init <= '0';
						ef_add <= '0';
						ef_final <= '0';
						datareg <= datareg;
						counterreg <= counterreg;
						enable_add <= '0';
						padding <= '0';
						select_padding_bit <= '0';
						done <= '1';
						next_state <= S_IDLE;
					else 
						counter <= (others=>'0');
						ef_init <= '0';
						ef_add <= '0';
						ef_final <= '0';
						datareg <= datareg;
						counterreg <= counterreg;
						enable_add <= '0';
						padding <= '0';
						select_padding_bit <= '0';
						done <= '0';
						next_state <= S_DONE;
					end if;
					
				when others =>
					counter <= (others => '0');
					ef_init <= '0';
					ef_add <= '0';
					ef_final <= '0';
					datareg <= datareg;
					counterreg <= counterreg;
					enable_add <= '1';
					padding <= '0';
					select_padding_bit <= '1';
					done <= '0';
					next_state <= S_IDLE;

			
			end case;
		end if;
	end process;



end Behavioral;

