library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity echo_block is
    Port (  chain_in : in  STD_LOGIC_VECTOR (511 downto 0);
            message_block : in  STD_LOGIC_VECTOR (1535 downto 0);
            rst, clk : in STD_LOGIC;
			start : in  STD_LOGIC;
            done : out  STD_LOGIC;
			bit_counter : in STD_LOGIC_VECTOR(63 downto 0);
			chain_outout: out STD_LOGIC_VECTOR(511 downto 0));
end echo_block;

architecture Behavioral of echo_block is

	--component declarations
	component  echo_round is
		Port (  round_in : in  STD_LOGIC_VECTOR (2047 downto 0);
				round_out : out  STD_LOGIC_VECTOR (2047 downto 0);
				round_counter : in  STD_LOGIC_VECTOR (2 downto 0);
				bit_counter:in std_logic_vector(63 downto 0));
	end component;

	--signal declarations
	signal bigrounds_counter: std_logic_vector(2 downto 0);
	signal startstate : STD_LOGIC_VECTOR (2047 downto 0);
	signal startstate_reg : STD_LOGIC_VECTOR (2047 downto 0);
	signal roundoutput : STD_LOGIC_VECTOR (2047 downto 0);
	signal registeroutput : STD_LOGIC_VECTOR (2047 downto 0);
	signal muxout : STD_LOGIC_VECTOR (2047 downto 0);
	signal muxsel : STD_LOGIC;
	signal regen : STD_LOGIC;
	signal reginen : STD_LOGIC;

	--states of the echo-fsm
	type FSM_state is (S_IDLE, S_LOOP, S_DONE);
	signal state: FSM_state;

begin

	--componenten instances
	inst_BSW: echo_round 
		Port map(muxout, roundoutput, bigrounds_counter, bit_counter);

	----------------------------------------------------------------registertje maken om de input vast te zetten bij start!
	--create the ECHO state
	startstate <= chain_in & message_block;

	--mux to select startstate or previous output
	mux: process(muxsel, startstate, registeroutput)
	begin	
		if muxsel='0' then
			muxout<=startstate;
		else
			muxout<=registeroutput;
		end if;
	end process;
	
	--register to save the inputstate
	reg1: process(clk, reginen, roundoutput)
	begin	
		if clk'event and clk='1' then
			if reginen = '1' then
				startstate_reg<=startstate;
			end if;
		end if;
	end process;
	
	--register to save the roundoutput
	reg2: process(clk, regen, roundoutput)
	begin	
		if clk'event and clk='1' then
			if regen = '1' then
				registeroutput<=roundoutput;
			end if;
		end if;
	end process;
	
	
	--the fsm
	fsm: process(rst, clk)
	begin
		if rst = '0' then
			bigrounds_counter <= "000";
			done <= '0';
			muxsel <= '0';
			regen <= '0';
			reginen <= '0';
			state <= S_IDLE;
			
		elsif clk'event and clk = '1' then
			case state is
				when S_IDLE =>
					if start = '1' then
						bigrounds_counter <="000";
						done <= '0';
						muxsel <= '0';
						regen <= '1';
						reginen <= '1';
						state <= S_LOOP;
					else
						bigrounds_counter <="000";
						done <= '0';
						muxsel <= '0';
						regen <= '0';
						reginen <= '0';
						state <= S_IDLE;
					end if;
					
				when S_LOOP =>
					if bigrounds_counter = "111" then
						bigrounds_counter <="000";
						done <= '0';
						muxsel <= '0';
						regen <= '0';
						reginen <= '0';
						state <= S_DONE;
					else
						bigrounds_counter <= bigrounds_counter+1;
						done <= '0';
						muxsel <= '1';
						regen <= '1';
						reginen <= '0';
						state <= S_LOOP;
					end if;
					
				when S_DONE =>
					if start = '1' then
						bigrounds_counter <="000";
						done <= '1';
						muxsel <= '0';
						regen <= '0';
						reginen <= '0';
						state <= S_DONE;
					else
						bigrounds_counter <="000";
						done <= '1';
						muxsel <= '0';
						regen <= '0';
						reginen <= '0';
						state <= S_IDLE;
					end if;
					
				when others =>
					bigrounds_counter <="000";
					done <= '0';
					muxsel <= '0';
					regen <= '0';
					reginen <= '0';
					state <= S_IDLE;
					
			end case;
		end if;
	end process;
	
	--calculate output (only correct for 256 bit!!)
	chain_outout(511 downto 384) <= startstate_reg(2047 downto 1920) XOR startstate_reg(1535 downto 1408) XOR startstate_reg(1023 downto 896) XOR startstate_reg(511 downto 384) XOR registeroutput(2047 downto 1920) XOR registeroutput(1535 downto 1408) XOR registeroutput(1023 downto 896) XOR registeroutput(511 downto 384);
	chain_outout(383 downto 256) <= startstate_reg(1919 downto 1792) XOR startstate_reg(1407 downto 1280) XOR startstate_reg(895 downto 768) XOR startstate_reg(383 downto 256) XOR registeroutput(1919 downto 1792) XOR registeroutput(1407 downto 1280) XOR registeroutput(895 downto 768) XOR registeroutput(383 downto 256) ;
	chain_outout(255 downto 128) <= startstate_reg(1791 downto 1664) XOR startstate_reg(1279 downto 1152) XOR startstate_reg(767 downto 640) XOR startstate_reg(255 downto 128) XOR registeroutput(1791 downto 1664) XOR registeroutput(1279 downto 1152) XOR registeroutput(767 downto 640) XOR registeroutput(255 downto 128) ;
	chain_outout(127 downto 0) <= startstate_reg(1663 downto 1536) XOR startstate_reg(1151 downto 1024) XOR startstate_reg(639 downto 512) XOR startstate_reg(127 downto 0) XOR registeroutput(1663 downto 1536) XOR registeroutput(1151 downto 1024) XOR registeroutput(639 downto 512) XOR registeroutput(127 downto 0) ;
	
end Behavioral;

