-- *********************************************
--
-- Author : Gert Kloeck
--
-- Katholieke Universiteit Leuven
-- Department of Electrical Engineering
-- Control Systems and Industrial Cryptography Research group
--
-- Advisors : Nele Mentens, Lejla Batina, Siddika Berna Ors
--
-- File : mixcolumn.vhd
--
-- Last Modified : 15/03/2004
--
-- Description :
-- 
-- This component implements the MixColumn operation as decribed in
-- the AES Standard Encryption Algorythm.	 It lineary combines
-- columns of the state.
--
-- Signals :
--
-- MC_in : INPUT 32bit std_logic_vector
-- MC_out : OUTPUT 32bit std_logic_vector
--
-- ********************************************


library ieee;
use ieee.std_logic_1164.all;

entity MixColumn is 
	port (	MC_in : in std_logic_vector (31 downto 0);
		MC_out : out std_logic_vector(31 downto 0));
end entity;


architecture MixColumn_arch of MixColumn is

signal din1, din2, din3, din4, dout1, dout2, dout3, dout4 : std_logic_vector(7 downto 0);
  
signal tmp1 : std_logic_vector(7 downto 0);
signal tmp2 : std_logic_vector(7 downto 0);
signal tmp3 : std_logic_vector(7 downto 0);
signal tmp4 : std_logic_vector(7 downto 0);

signal byte5 : std_logic_vector(7 downto 0);
signal byte6 : std_logic_vector(7 downto 0);
signal byte7 : std_logic_vector(7 downto 0);
signal byte8 : std_logic_vector(7 downto 0);

signal byte9  : std_logic_vector(7 downto 0);
signal byte10 : std_logic_vector(7 downto 0);
signal byte11 : std_logic_vector(7 downto 0);
signal byte12 : std_logic_vector(7 downto 0);
  
begin  -- rtl

-- mapping of the bytes of the column to signals for ease of use

  din1 <= MC_in(31 downto 24);
  din2 <= MC_in(23 downto 16);
  din3 <= MC_in(15 downto 8);
  din4 <= MC_in(7 downto 0);

--  leftshift of all the inputs.

  tmp1(7 downto 1) <= din1(6 downto 0) ;
  tmp1(0) <= '0' ;
  tmp2(7 downto 1) <= din2(6 downto 0) ;
  tmp2(0) <= '0' ;
  tmp3(7 downto 1) <= din3(6 downto 0) ;
  tmp3(0) <= '0' ;
  tmp4(7 downto 1) <= din4(6 downto 0) ;
  tmp4(0) <= '0' ;

  l1: process (din1,tmp1)
  begin  -- process l1
    if din1(7) = '1' then
      byte5 <= tmp1 xor "00011011" ;
    else
      byte5 <= tmp1 ;
    end if;
  end process l1;

  l2: process (din2,tmp2)
  begin  -- process l2
    if din2(7) = '1' then
      byte6 <= tmp2 xor "00011011" ;
    else
      byte6 <= tmp2 ;
    end if;
  end process l2;

  l3: process (din3,tmp3)
  begin  -- process l3
    if din3(7) = '1' then
      byte7 <= tmp3 xor "00011011" ;
    else
      byte7 <= tmp3 ;
    end if;
  end process l3;
  
  l4: process (din4,tmp4)
  begin  -- process l4
    if din4(7) = '1' then
      byte8 <= tmp4 xor "00011011" ;
    else
      byte8 <= tmp4 ;
    end if;
  end process l4;

  byte9  <= din1 xor byte5 ;
  byte10 <= din2 xor byte6 ;
  byte11 <= din3 xor byte7 ;  
  byte12 <= din4 xor byte8 ;

  dout1 <= byte5 xor byte10 xor din3 xor din4 ;
  dout2 <= byte6 xor byte11 xor din4 xor din1 ;
  dout3 <= byte7 xor byte12 xor din1 xor din2 ;
  dout4 <= byte8 xor byte9  xor din2 xor din3 ;

  MC_out <= dout1 & dout2 & dout3 & dout4;

end;
