-- in deze blok zitten 2 counters(een voor aantal boodschappen,een voor aantal 
--binnengestroomde bits), message padding, een 32bitBuffer
-- 
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Hamsi_Buffer is
	port (	input,init,end_last_message, clk, rst,enable,NLP_ready: in std_logic;
			 output: out std_logic_vector(31 downto 0);
			 done,chaining_value_select,data_ready,init_ready,last_block: out std_logic);
end Hamsi_Buffer;

architecture Buffer_arch of Hamsi_Buffer is


component FFBuffer is
	port (	buffer_in, clk, rst,enable_buffer: in std_logic;
			 buffer_out: out std_logic_vector(31 downto 0));
end component;

component Counter6bit is
    port(enable_6bc, clk, rst: in std_logic;
         enable_buffer: out std_logic;
         output : out std_logic_vector(5 downto 0)
        );
end component;
   
component MessagePadding is
    port(enable_MP, clk, rst,preset: in std_logic;
         output_MP: out std_logic
        );
end component;
   
component MessageCounter is
    port(enable_MC, clk, rst: in std_logic;
         MC_out: out std_logic_vector(63 downto 0)
        );
end component;

component enable_FSM is
	port (	Enable_in,clk,rst: in std_logic;
			 enable_out: out std_logic);
end component;

component Data_ready_fsm is
	port (	data_ready_in,clk,rst: in std_logic;
			 Data_ready: out std_logic);
end component;

type my_state is (waiting0,waiting1,waiting2,waiting3,done_state,waiting,stop_running,idle,init_State,running,messagepadding_State,messagelength1,messagelength2);

signal state :my_state;
signal input_buffer,input_mp,data_ready_length: std_logic; 
signal enable_messagecounter,enable_buffer,enable_bufferinput,enable_out,enable_6bitcounter,enable_mp: std_logic;
signal preset: std_logic; 
signal rst_counter,rst_6bc,data_ready_6bc,DR_6BC,init_ready_int: std_logic;
signal output_ml1,output_ml2,output_reg,output_buffer:std_logic_vector(31 downto 0);
signal output_mc:std_logic_vector(63 downto 0);
signal BC_Out : std_logic_Vector(5 downto 0);

begin
inst_Data_ready_fsm_6bc: Data_ready_fsm
	port map (	data_ready_6bc,clk,rst,DR_6bc);

data_ready <= DR_6bc or data_ready_length ;

output_ml1 <= output_mc(63 downto 32);
output_ml2 <= output_mc(31 downto 0);




inst_32bitBuffer: FFBuffer
      port map(input_buffer,clk,rst,enable_buffer,output_buffer);
 
inst_messagecounter: MessageCounter
      port map(enable_messagecounter,clk,rst,output_mc);

inst_messagepadding: MessagePadding
      port map(enable_mp,clk,rst,preset,input_mp);

inst_6bitcounter: Counter6bit
      port map(enable_6bitcounter,clk,rst_6bc,enable_bufferinput,BC_out);
          
inst_enable_FSM: enable_FSM
      port map(enable,clk,rst,enable_out);


       
data_ready_6bc  <= not(enable_bufferinput);
enable_buffer <= enable_out and enable_bufferinput;
enable_6bitcounter <= enable_out and enable_bufferinput;
enable_mp <= end_last_message and enable_buffer;
rst_6bc <= rst or rst_counter;
-----------------------------------------------------------------------
------------------------------------------------------------------------------
------------------------------------process voor enalbe mc 
process(state,data_ready_6bc,enable_buffer)
begin
	case state is 
		when messagelength1 =>
			enable_messagecounter <= '0';
		when messagelength2 =>
			enable_messagecounter <= '0';
		when waiting0 =>
			enable_messagecounter <= '0';
		when waiting1 =>
			enable_messagecounter <= '0';
		when waiting2 =>
			enable_messagecounter <= '0';
		when waiting3 =>
			enable_messagecounter <= '0';
		when done_State =>
			enable_messagecounter <= '0';
		when messagepadding_State =>
			enable_messagecounter <= '0';
		when others =>
			enable_messagecounter <= enable_buffer; 
	end case;
end process;	

-- in het volgende deel wordt bepaald wat er aan de ingang van de 32bitBuffer wordt gehangen.
-- nl de normale input van de powerpc of de blok MessagePadding
process(clk,rst)
begin
	if rst = '1' then
	output_reg <= (others => '0');
	elsif rising_edge(clk) then
		if data_ready_6Bc = '1' then
		  output_reg <= output_buffer;
				
	   else output_reg <= output_reg;
		end if;
	end if;
end process;
		  
process(end_last_message,input_mp,input)
    begin
        if end_last_message = '1' then
            input_buffer <= input_mp;
        else
            input_buffer <= input;
    end if;
end process;


process(output_ml1,output_ml2,state,output_reg)
begin
	case state is 
		when messagelength1 =>
			output <= output_ml1;
		when waiting2 =>
			output <= output_ml1;
		when waiting3 =>
			output <= output_ml1;
		when messagelength2 =>
			output <= output_ml2;
		when done_state =>
		
			output <= output_ml2;
	
		when others => output <= output_Reg;
	end case;
end process;
-------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------
cv_select_proc : process(state, output_ml1, output_ml2)
begin 
	case state is
		    when waiting0 =>
				chaining_Value_select <= '1';
		    when waiting1 =>
				chaining_Value_select <= '1';
		    when waiting2 =>
				chaining_Value_select <= '1';
		    when waiting3 =>
				chaining_Value_select <= '1';
		    when done_state =>
				chaining_Value_select <= '1';
		    when waiting =>
				chaining_Value_select <= '1';
		    when messagepadding_State =>
				chaining_Value_select <= '1';
		    when messagelength1 =>
				chaining_Value_select <= '1';
		    when messagelength2 =>
				chaining_Value_select <= '1';
			 when others =>		  
		     if output_ml1 & output_ml2 > 32 then
				  chaining_Value_select <= '1';
			  else 
			      chaining_Value_select <= '0';
			  end if;
    end case;       
	 
end process;



init_ready <= init_Ready_int;	    	
fsm: process(rst,clk)
begin 
   	if rst = '1' then
			  preset <= '0';
			  init_ready_int <= '0';		
			  done <= '0';
			  data_ready_length <= '0';
			  state <= idle;
			  rst_counter <= '0';
			  
			  last_block <= '0';

	 elsif rising_edge(clk) then   
			case state is
				when idle =>	
					if init = '1' AND init_ready_int = '0' then
				      preset <= '1';
				      init_ready_int <= '1';
				      done <= '0';
						data_ready_length <= '0';
						state <= init_State;
						rst_counter <= '0';
						
						last_block <= '0';
					else--if init_ready_int ='1' then
						preset <= '1'; -- asynchrone preset
						init_ready_int<='1';
						done <= '0';
						data_ready_length <= '0';
						state <= running;
						rst_counter <= '0';
						
						last_block <= '0';
					end if;
		
				when init_State =>
					   preset <= '0';
					   init_ready_int <= '1';
					   done <= '0';
						data_ready_length <= '0';
					   state <= running;
						rst_counter <= '0';
						
						last_block <= '0';

				when running =>
					if end_last_message = '1' then
					   preset <= '0'; 
					   init_ready_int <= '1';
					   done <= '0';
						data_ready_length <= '0';
					   state <= messagepadding_State;
						rst_counter <= '0';
						
						last_block <= '0';
					elsif data_ready_6bc='1' then
					   preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						data_ready_length <= '0';
				      state <= running;
						rst_counter <= '1';
						
						last_block <= '0';
					else 
					   preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						data_ready_length <= '0';
				      state <= running;
						rst_counter <= '0';
						
						last_block <= '0';
					end if;
					
				when messagepadding_State =>
				   if data_Ready_6bc = '1' then 
				      preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						data_ready_length <= '1';
				      state <=waiting0;
						rst_counter <= '0';
						last_block <= '0';
						
				   else
				      preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						data_ready_length <= '0';
				      state <= messagepadding_state;
						rst_counter <= '0';
						last_block <= '0';
						
				   end if;
				when waiting0 =>
						preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						data_ready_length <= '0';
				      state <= waiting1;
						rst_counter <= '0';
						last_block <= '0';
						
				when waiting1 =>
					if nlp_ready = '1' then
						preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						data_ready_length <= '0';
				      state <= messagelength1;
						rst_counter <= '0';
						last_block <= '0';
						
					else 
						preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						data_ready_length <= '0';
				      state <= waiting1;
						rst_counter <= '0';
						last_block <= '0';
						
					end if;
				when messagelength1 =>
						preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
					   state <= waiting2;
						rst_counter <= '0';
						
						last_block <= '0';
						data_ready_length <= '1';
				when waiting2 =>
						preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						data_ready_length <= '0';
				      state <= waiting3;
						rst_counter <= '0';
						last_block <= '0';
						
				when waiting3 =>
					if NLP_ready = '1' then
						preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						data_ready_length <= '0';
				      state <= messagelength2;
						rst_counter <= '0';
						last_block <= '0';
						
					else
						preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						data_ready_length <= '0';
				      state <= waiting3;
						rst_counter <= '0';
						last_block <= '0';
						
					end if;
				when messagelength2 =>
						preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
				      state <= done_state;
						rst_counter <= '0';
						
						last_block <= '1';
						data_ready_length <= '1';	
				when done_state =>
						preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '1';
						last_block <= '1';
						data_ready_length <= '0';
				      state <= done_state;
						rst_counter <= '1';
						
					
				when others =>
				      preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						last_block <= '0';
						data_ready_length <= '0';
				      state <= idle;
						rst_counter <= '0';
						
			end case;
		end if;
	end process;     


	
end Buffer_arch;
	
	
