
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity NLP is
    port(input: in std_logic_vector(511 downto 0);            
			Counter_value: in std_logic_vector(4 downto 0);			
			last_nlp: in std_logic;
         output: out std_logic_vector(511 downto 0)
			
        );
end NLP;

architecture Behavioral of NLP is
   

component Xor_poort is
   port( Xor_in :in std_logic_vector( 511 downto 0 );
         Xor_out :out std_logic_vector( 511 downto 0 );
         Counter_value: in std_logic_vector(4 downto 0);
         Last_NLP: in std_logic);
end component;

component Sbox is
   port( Sbox_in :in std_logic_vector( 3 downto 0 );
         Sbox_out :out std_logic_vector( 3 downto 0 ));
end component;

component Diffusion is
   port( Dif_input :in std_logic_vector( 127 downto 0 );
         Dif_output :out std_logic_vector( 127 downto 0 ));
         
end component;



component Truncation is
    port(Trunc_in: in std_logic_vector(511 downto 0);
         Trunc_out: out std_logic_vector(255 downto 0));
end component; 
  
type my_state is (waiting,last_partial_NLP,partial_NLP,done_state);
signal state : my_state;
signal input_xor,tmp_reg ,in_Dif,in_sbox: std_logic_vector(511 downto 0);
signal out_Xor,out_Sbox,out_Dif,temp_sbox,temp_dif : std_logic_vector(511 downto 0);
signal out_Counter : std_logic_vector(4 downto 0);
signal out_Trunc : std_logic_vector(255 downto 0);

signal enable_counter,rst_counter,reset_counter,last_nlp_done,load_tmp: std_logic;
signal output_reg : std_logic_vector(255 downto 0);
signal dif1,dif2,dif3,dif4,out_dif1,out_dif2,out_dif3,out_dif4 : std_logic_Vector(127 downto 0);
signal a0,a1,a2,a3,b0,b1,b2,b3,c0,c1,c2,c3,d0,d1,d2,d3,
       out_a0,out_a1,out_a2,out_a3,out_b0,out_b1,out_b2,
		 out_b3,out_c0,out_c1,out_c2,out_c3,out_d0,out_d1,out_d2,out_d3 : std_logic_vector(31 downto 0);
signal row1,row2,row3,row4,srow1,srow2,srow3,srow4: std_logic_Vector(127 downto 0);

begin

inst_Xor : Xor_poort
      port map( input, out_xor, out_counter,last_NLP);

out_counter <= counter_Value;
--Sbox
--eerst bits op opdelen per rij dan per 4 bij elkaar horende bits samenzetten en achter elkaar zetten
--hierna doorheen sbox en naderhand weer ontbinden. sbox werkt namelijk per kollom (4bits) 
--indeling voor Sbox:




row1 <= out_xor(511 downto 384);
row2 <= out_xor(383 downto 256);
row3 <= out_xor(255 downto 128);
row4 <= out_xor(127 downto 0);

gen_input_sbox : for i in 127 downto 0 generate
	in_sbox(4*i+3 downto 4*i) <= row4(i) & row3(i) & row2(i)& row1(i);
end generate;

 gen_Sbox: for i in 127 downto 0 generate
 inst_Sbox : Sbox
  port map(in_sbox(4*i+3 downto 4*i),temp_sbox(4*i+3 downto 4*i));
end generate;

gen_rows_out_of_sbox : for i in 127 downto 0 generate
	srow4(i) <= temp_sbox(4*i+3);
	srow3(i) <= temp_sbox(4*i+2);
	srow2(i) <= temp_sbox(4*i+1);
	srow1(i) <= temp_sbox(4*i);	
end generate;

out_sbox <= srow1 & srow2 & srow3 & srow4;
-- einde Sbox

--Diffusie
-- eerst matrix opdelen via diagonalen, deze doorheen diffusie, weer ontbinden en samenvoegen

--opdeling voor diffusie
a0 <= out_sbox(511 downto 480);
b0 <= out_sbox(479 downto 448);
c0 <= out_sbox(447 downto 416);
d0 <= out_sbox(415 downto 384);
d1 <= out_sbox(383 downto 352);
a1 <= out_sbox(351 downto 320);
b1 <= out_sbox(319 downto 288);
c1 <= out_sbox(287 downto 256);
c2 <= out_sbox(255 downto 224);
d2 <= out_sbox(223 downto 192);
a2 <= out_sbox(191 downto 160);
b2 <= out_sbox(159 downto 128);
b3 <= out_sbox(127 downto 96);
c3 <= out_sbox(95 downto 64);
d3 <= out_sbox(63 downto 32);
a3 <= out_sbox(31 downto 0);










dif1 <= a0 & a1 & a2 & a3;
dif2 <= b0 & b1 & b2 & b3;
dif3 <= c0 & c1 & c2 & c3;
dif4 <= d0 & d1 & d2 & d3;
in_dif <= dif1 & dif2 & dif3 & dif4;

--diffusie zelf

gen_Diffusion: for i in 3 downto 0 generate   -- is niet juist aangesloten, moet via diagonalen... opgelost
   inst_Diffusion: Diffusion
   
      port map(in_Dif(128*i+127 downto 128*i),temp_Dif(128*i+127 downto 128*i));
end generate;

--terug ontbinden
out_dif1 <= temp_dif(511 downto 384);
out_dif2 <= temp_dif(383 downto 256);
out_dif3 <= temp_dif(255 downto 128);
out_dif4 <= temp_dif(127 downto 0);

out_a0 <= out_dif1(127 downto 96); 
out_a1 <= out_dif1(95 downto 64); 
out_a2 <= out_dif1(63 downto 32); 
out_a3 <= out_dif1(31 downto 0); 
out_b0 <= out_dif2(127 downto 96); 
out_b1 <= out_dif2(95 downto 64); 
out_b2 <= out_dif2(63 downto 32); 
out_b3 <= out_dif2(31 downto 0); 
out_c0 <= out_dif3(127 downto 96); 
out_c1 <= out_dif3(95 downto 64); 
out_c2 <= out_dif3(63 downto 32); 
out_c3 <= out_dif3(31 downto 0); 
out_d0 <= out_dif4(127 downto 96); 
out_d1 <= out_dif4(95 downto 64); 
out_d2 <= out_dif4(63 downto 32); 
out_d3 <= out_dif4(31 downto 0); 

out_dif <=  out_a0 & out_b0 & out_c0 & out_d0
          & out_d1 & out_a1 & out_b1& out_c1 
			 & out_c2 & out_d2 & out_a2 & out_b2 
			 & out_b3 & out_c3 & out_d3 & out_a3 ;

-- einde diffusie



output <= out_dif;

       
       




end behavioral;

 
