library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Lane_full is port(
	clk, rst, init, add, final: in std_logic;
	done: out std_logic;
	data_in: in std_logic_vector(511 downto 0);
	data_bits: in std_logic_vector(9 downto 0);
	hash_out: out std_logic_vector(255 downto 0));
end Lane_full;

architecture Behavioral of Lane_full is

	component Lane_block is
		Port ( 	chaining_value : in  STD_LOGIC_VECTOR (255 downto 0);
				message_block : in  STD_LOGIC_VECTOR (511 downto 0);
				rst, clk : in STD_LOGIC;
				start : in  STD_LOGIC;
				done : out  STD_LOGIC;
				bit_counter : in STD_LOGIC_VECTOR(63 downto 0);
				hash_out: out STD_LOGIC_VECTOR(255 downto 0)
			  );
	end component;

	type FSMstate is (S_IDLE, S_INIT, S_ADD, S_PRE_FINAL, S_PRE_2_FINAL, S_FINAL);
	signal next_state: FSMstate := S_IDLE;
	
	signal IV : std_logic_vector(255 downto 0);
	signal hashval : std_logic_vector(255 downto 0);
	signal chain_mux : std_logic_vector(255 downto 0);
	signal data_mux : std_logic_vector(511 downto 0);
	signal last_data : std_logic_vector(511 downto 0);
	signal sel_chain : std_logic;
	signal sel_data : std_logic;
	signal lane_start : std_logic;
	signal lane_done : std_logic;
	signal lane_rst : std_logic;
	signal chaining_reg : std_logic_vector(255 downto 0);
	signal data_reg : std_logic_vector(511 downto 0);
	signal bitcounter : std_logic_vector(63 downto 0);

begin

	lane_function: Lane_block port map(chaining_reg, data_reg, rst, clk, lane_start, lane_done, bitcounter, hashval);

	IV <= x"be292e17bb541ff2fe54b6f730b1c96a7b2592688539bdf397c4bdd649763fb8"; --zet de juiste waarde in IV
	last_data(511 downto 440) <= ("00000000" & bitcounter);
	
	last_data(439 downto 0) <= (others => '0');
	hash_out <= hashval;


	mux_chain: process(sel_chain, IV, hashval)
	begin
		if sel_chain='1' then	
			chain_mux <= hashval;
		else
			chain_mux <= IV;
		end if;
	end process;
	
	mux_data: process(sel_data, last_data, data_in)
	begin
		if sel_data='1' then	
			data_mux <= last_data;
		else
			data_mux <= data_in;
		end if;
	end process;



	FSM_FULL: process(rst,clk)

	begin

		if rst = '0' then --laag actieve reset
			lane_rst <= '0';
			lane_start <= '0';
			sel_chain <= '0';
			sel_data <= '0';
			chaining_reg <= chaining_reg;
			data_reg <= data_mux;
			done <= '0';
			bitcounter <= (others => '0');
			next_state <= S_IDLE;

		elsif clk'event and clk = '1' then
			case next_state is
				when S_IDLE =>
					if init = '1' and add = '0' and final = '0' then 
						lane_rst <= '1';
						lane_start <= '0';
						sel_chain <= '0';
						sel_data <= '0';
						chaining_reg <= chain_mux;
						data_reg <= data_reg;
						done <= '0';
						bitcounter <= (others => '0');
						next_state <= S_INIT;
						
					elsif init = '0' and add = '1' and final = '0' then 
						lane_rst <= '1';
						lane_start <= '0';
						sel_chain <= sel_chain;
						sel_data <= '0';
						chaining_reg <= chain_mux;
						data_reg <= data_mux;
						done <= '0';
						bitcounter <= bitcounter + data_bits;
						next_state <= S_ADD;
						
					elsif init = '0' and add = '0' and final = '1' then 
						lane_rst <= '1';
						lane_start <= '0';
						sel_chain <= '1';
						sel_data <= '1';
						chaining_reg <= chain_mux;
						data_reg <= data_reg;
						done <= '0';
						bitcounter <= bitcounter;
						next_state <= S_PRE_FINAL;
						
					else
						lane_rst <= '1';
						lane_start <= '0';
						sel_chain <= sel_chain;
						sel_data <= sel_data;
						chaining_reg <= chaining_reg;
						data_reg <= data_mux;
						done <= '0';
						bitcounter <= bitcounter;
						next_state <= S_IDLE;
						
					end if;
				
				when S_INIT =>
						lane_rst <= '1';
						lane_start <= '0';
						sel_chain <= '0';
						sel_data <= '0';
						chaining_reg <= chaining_reg;
						data_reg <= data_mux;
						done <= '1';
						bitcounter <= (others => '0');
						next_state <= S_IDLE;

				
				when S_ADD =>
					if lane_done = '1' then 
						lane_rst <= '1';
						lane_start <= '0';
						sel_chain <= '1';
						sel_data <= '0';
						chaining_reg <= chaining_reg;
						data_reg <= data_mux;
						done <= '1';
						bitcounter <= bitcounter;
						next_state <= S_IDLE;
					
					else
						lane_rst <= '1';
						lane_start <= '1';
						sel_chain <= sel_chain;
						sel_data <= '0';
						chaining_reg <= chaining_reg;
						data_reg <= data_mux;
						done <= '0';
						bitcounter <= bitcounter;
						next_state <= S_ADD;
					end if;
							
				
				when S_PRE_FINAL =>
						lane_rst <= '1';
						lane_start <= '1';
						sel_chain <= '1';
						sel_data <= '1';
						chaining_reg <= chaining_reg;
						data_reg <= data_mux;
						done <= '0';
						bitcounter <= bitcounter;
						next_state <= S_PRE_2_FINAL;
				
				when S_PRE_2_FINAL =>
						lane_rst <= '1';
						lane_start <= '1';
						sel_chain <= '1';
						sel_data <= '1';
						chaining_reg <= chaining_reg;
						data_reg <= data_reg;
						done <= '0';
						bitcounter <= (others=>'0');
						next_state <= S_FINAL;
						
					
				when S_FINAL =>
						if lane_done = '1' then 
							lane_rst <= '1';
							lane_start <= '0';
							sel_chain <= '0';
							sel_data <= '0';
							chaining_reg <= chaining_reg;
							data_reg <= data_reg;
							done <= '1';
							bitcounter <= bitcounter;
							next_state <= S_IDLE;
						
						else
							lane_rst <= '1';
							lane_start <= '1';
							sel_chain <= '1';
							sel_data <= '1';
							chaining_reg <= chaining_reg;
							data_reg <= data_reg;
							done <= '0';
							bitcounter <= bitcounter;
							next_state <= S_FINAL;
						
						end if;
						
				when others =>
					lane_rst <= '0';
					lane_start <= '0';
					sel_chain <= '0';
					sel_data <= '0';
					chaining_reg <= chaining_reg;
					data_reg <= data_mux;
					done <= '0';
					bitcounter <= (others => '0');
					next_state <= S_IDLE;
			
			end case;
		end if;
	end process;



end Behavioral;

