
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Lane_full_buffer is port(
	bits, clk, rst, init, add, final: in std_logic;
	done: out std_logic;
	data: in std_logic_vector(7 downto 0);
	hashval: out std_logic_vector(255 downto 0));
end Lane_full_buffer;

architecture Behavioral of Lane_full_buffer is

	component Lane_full is 
		port(	clk, rst, init, add, final: in std_logic;
				done: out std_logic;
				data_in: in std_logic_vector(511 downto 0);
				data_bits: in std_logic_vector(9 downto 0);
				hash_out: out std_logic_vector(255 downto 0));
	end component;
	
	component shiftreg512 is
		port(	shiftin: in STD_LOGIC_VECTOR(7 downto 0);
				clk: in STD_LOGIC;
				rst: in STD_LOGIC;
				parallel_out: out STD_LOGIC_VECTOR(511 downto 0);
				bits : in std_logic;
				enable : in std_logic);
	end component;

	type FSMstate is (S_IDLE, S_RESET, S_PRE_FINAL, S_FINAL, S_DONE);
	signal next_state: FSMstate := S_IDLE;
	
	signal lf_init: std_logic;
	signal lf_add: std_logic;
	signal lf_add_without_delay: std_logic;
	signal lf_final: std_logic;
	signal lf_done: std_logic;
	signal mux_null: std_logic_vector(7 downto 0);
	signal mux_sel: std_logic;
	signal mux_out: std_logic_vector(7 downto 0);
	signal padding: std_logic;
	
	signal shift_clk: std_logic;
	
	signal shift_parallel_out: std_logic_vector(511 downto 0);
	signal datablock: std_logic_vector(511 downto 0);
	signal counter: std_logic_vector(9 downto 0);
	signal counterpass: std_logic_vector(9 downto 0);
	signal paddingcounter: std_logic_vector(9 downto 0);
	signal bitspass: std_logic;
	signal paddingbits: std_logic;
	signal regenable: std_logic;

begin

	inst_lane_full: Lane_full port map(clk, rst, lf_init, lf_add, lf_final, lf_done, datablock, counterpass, hashval);
	inst_shiftreg512: shiftreg512 port map(mux_out, shift_clk, rst,shift_parallel_out,bitspass,regenable);

	mux_null<="00000000";
	regenable<=(add or padding);
	
	process_mux_padding: process(bits, paddingbits,padding)
	begin
		if padding = '0' then
			bitspass <= bits;
		else
			bitspass <= paddingbits;
		end if;
	end process;	
	

	process_mux: process(data, mux_null, mux_sel)
	begin
		if mux_sel = '0' then
			mux_out <= data;
		else
			mux_out <= mux_null;
		end if;
	end process;


	puls_delay: process(clk)
	begin
		if clk'event and clk='1' then
			lf_add<=lf_add_without_delay;
		end if;
	end process;


	shiftprocess: process(clk)
	begin
		if clk'event and clk='1' then
			if init='1' then
				counter<=(others=>'0');
				datablock <= datablock;	
				counterpass<=counterpass;
				lf_add_without_delay<='0';
			
			elsif counter="1000000000" then
				datablock <= shift_parallel_out;
				lf_add_without_delay<='1';
				
				if add='1' then	
					counterpass<=counter;
					if bits ='0' then
						counter<="0000000001";
					else
						counter<="0000001000";
					end if;
				else
					counterpass<=counterpass;
					counter<=(others=>'0');
				end if;
			
			elsif paddingcounter="0111111111" then
					datablock <= shift_parallel_out;	
					counterpass<=counter;
					counter<=counter;
					lf_add_without_delay<='1';
			
			elsif add='1' then
				counterpass<=counterpass;
				if bits ='0' then
					counter<=counter+1;
				else
					counter<=counter+8;
				end if;
				datablock <= datablock;
				lf_add_without_delay<='0';
			
			else
					counterpass<=counterpass;
					counter<=counter;
					datablock<=datablock;
					lf_add_without_delay<='0';
			end if;
		end if;
	end process;
	shift_clk<=(add or padding) and clk;
	


	FSM_FULL_BUFFER: process(rst,clk)

	begin

		if rst = '0' then --laag actieve reset
			paddingcounter<=(others=>'0');
			mux_sel<='0';
			lf_init<='0';
			lf_final<='0';
			padding<='0';
			done<='0';
			paddingbits<='0';
			next_state <= S_IDLE;

		elsif clk'event and clk = '1' then
			case next_state is
				when S_IDLE =>
					if init = '1' and add = '0' and final = '0' then 
						paddingcounter<=(others=>'0');
						mux_sel<='0';
						lf_init<='1';
						lf_final<='0';
						padding<='0';
						done<='0';
						paddingbits<='0';
						next_state <= S_IDLE;
							
					elsif init = '0' and add = '0' and final = '1' then 
						paddingcounter<=counter;
						mux_sel<='1';
						lf_init<='0';
						lf_final<='0';
						padding<='0';
						done<='0';
						if counter<"0111110000" then
							paddingbits<='1';
						else
							paddingbits<='0';
						end if;
						next_state <= S_PRE_FINAL;
						
					else
						paddingcounter<=(others=>'0');
						mux_sel<='0';
						lf_init<='0';
						lf_final<='0';
						padding<='0';
						done<='0';
						paddingbits<='0';
						next_state <= S_IDLE;
						
					end if;
				
				when S_PRE_FINAL =>
					if paddingcounter="0111111111" then
						paddingcounter<=paddingcounter;
						mux_sel<='1';
						lf_init<='0';
						lf_final<='0';
						padding<='0';
						done<='0';
						paddingbits<='0';
						next_state <= S_FINAL;
					else
						if paddingbits ='0' then
							paddingcounter<=paddingcounter+1;
						else
							paddingcounter<=paddingcounter+8;
						end if;
						
						mux_sel<='1';
						lf_init<='0';
						lf_final<='0';
						padding<='1';
						done<='0';
						
						if paddingcounter<"0111110000" then
							paddingbits<='1';
						else
							paddingbits<='0';
						end if;
						
						next_state <= S_PRE_FINAL;
					end if;
	
				when S_FINAL =>
					if lf_done='1' then
						paddingcounter<=paddingcounter;
						mux_sel<='0';
						lf_init<='0';
						lf_final<='1';
						padding<='0';
						done<='0';
						paddingbits<='0';
						next_state <= S_DONE;
					else
						paddingcounter<=(others=>'0');
						mux_sel<='0';
						lf_init<='0';
						lf_final<='0';
						padding<='0';
						done<='0';
						paddingbits<='0';
						next_state <= S_FINAL;
					end if;
					
				when S_DONE =>
					if lf_done='1' then
						paddingcounter<=paddingcounter;
						mux_sel<='0';
						lf_init<='0';
						lf_final<='0';
						padding<='0';
						done<='1';
						paddingbits<='0';
						next_state <= S_IDLE;
					else
						paddingcounter<=paddingcounter;
						mux_sel<='0';
						lf_init<='0';
						lf_final<='0';
						padding<='0';
						done<='0';
						paddingbits<='0';
						next_state <= S_DONE;
					end if;
					
				when others =>
					paddingcounter<=paddingcounter;
					mux_sel<='0';
					lf_init<='0';
					lf_final<='0';
					padding<='0';
					done<='0';
					paddingbits<='0';
					next_state <= S_IDLE;

			
			end case;
		end if;
	end process;



end Behavioral;

