----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:51:23 11/09/2007 
-- Design Name: 
-- Module Name:    top_AES - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity round is
    Port (  plaintext : in  STD_LOGIC_VECTOR (255 downto 0);
            ciphertext : out  STD_LOGIC_VECTOR (255 downto 0);
            rst, clk : in STD_LOGIC;
			start : in  STD_LOGIC;
            done : out  STD_LOGIC;
			bit_counter : in STD_LOGIC_VECTOR(63 downto 0);
			section_number : in STD_LOGIC_VECTOR(2 downto 0);
			rounds : in  STD_LOGIC
			  );
end round;

architecture Behavioral of round is

	component ByteSub is
		port( BS_in :in std_logic_vector( 7 downto 0 );
				BS_out :out std_logic_vector( 7 downto 0 )
	);
	end component;
	
	component ShiftRow is
		port (	shiftrow_in : in std_logic_vector(127 downto 0);
					shiftrow_out : out std_logic_vector(127 downto 0));
	end component;

	component MixColumn is 
		port (	MC_in : in std_logic_vector (31 downto 0);
					MC_out : out std_logic_vector(31 downto 0));
	end component;
	
	component addConstants is
		port( AC_in :in std_logic_vector( 31 downto 0 );
				AC_out :out std_logic_vector( 31 downto 0 );
				roundnumber :in std_logic_vector(8 downto 0));
	end component;
	
	
	component AddCounter is
		port(	addCounter_in: in std_logic_vector(255 downto 0);
				counter: in std_logic_vector(63 downto 0);
				addCounter_out: out std_logic_vector(255 downto 0);
				roundnumber_lastbit: in std_logic);
		end component;
	
	
	component swapcolumns is	port(
			sc_in1, sc_in2: in std_logic_vector(127 downto 0);
			sc_out1, sc_out2: out std_logic_vector(127 downto 0));
	end component;


	signal out_BS, out_SR, out_MC, out_ACts, out_ACtr, out_SC, out_mux_last: std_logic_vector(255 downto 0);
	signal en, last: std_logic;
	signal load2: std_logic:='0';
	signal counter: std_logic_vector(2 downto 0);
	signal ark, out_mux_begin1: std_logic_vector(255 downto 0);
	signal ark_reg: std_logic_vector(255 downto 0):=(others => '0');
	signal roundnumber: std_logic_vector(8 downto 0);
	signal counter_integer, section_number_integer: integer;
	signal plusval, section_multiplyer: integer;
	signal zeros: std_logic_vector(31 downto 0);
	signal in_SC: std_logic_vector(255 downto 0);
	
	type my_arr is array(7 downto 0) of std_logic_vector(8 downto 0);
	signal arr: my_arr;

	type my_state is (idle, state1, state2);
	signal state: my_state;
	
	--input via signals omdat anders number of IOB's overschreden wordt
	--, ciphertext
	--signal plaintext: std_logic_vector(255 downto 0);
	--signal bit_counter: std_logic_vector(63 downto 0);


begin

	mux_begin1: process(plaintext, ark_reg, load2)
	begin
		if load2 = '0' then
			out_mux_begin1 <= ark_reg;
		else
			out_mux_begin1 <= plaintext;
		end if;
	end process;

reg_ark: process(rst, clk)
	begin
		if rst = '0' then
			ark_reg <= (others => '0');
		elsif clk'event and clk = '1' then
			if en = '1' then
				ark_reg <= ark;
			else
				ark_reg <= ark_reg;
			end if;
		end if;
	end process;
		
	gen_ByteSub: for i in 31 downto 0 generate
	inst_ByteSub: ByteSub
		port map(out_mux_begin1(8*i+7 downto 8*i), out_BS(8*i+7 downto 8*i));
	end generate;


	gen_ShiftRow: for i in 1 downto 0 generate
	inst_ShiftRow: ShiftRow
		port map(out_BS(128*i + 127 downto 128*i), out_SR(128*i + 127 downto 128*i));
	end generate;

	gen_MC: for i in 7 downto 0 generate
		inst_MixColumn: MixColumn 
			port map(out_SR(32*i+31 downto 32*i), out_MC(32*i+31 downto 32*i));
	end generate;
	
	arr(7) <= roundnumber;
	arr(6) <= roundnumber + 1;
	arr(5) <= roundnumber + 2;
	arr(4) <= roundnumber + 3;
	arr(3) <= roundnumber + 4;
	arr(2) <= roundnumber + 5;
	arr(1) <= roundnumber + 6;
	arr(0) <= roundnumber + 7;
	
	
	zeros <= x"00000000";
	
	gen_ACts: for i in 7 downto 0 generate
		inst_AddConstants: addconstants
			port map(zeros, out_ACts(32*i+31 downto 32*i), arr(i));
	end generate;
	
	inst_ACtr: AddCounter
		port map(out_ACts(255 downto 0), bit_counter, out_ACtr(255 downto 0), roundnumber(3));

	
	in_SC<= out_MC XOR out_mux_last;
	
	
	--is niet meer mux last zoals in vorige implementatie!!
	mux_last: process(out_ACtr, last)
	begin
		if last = '0' then
			out_mux_last <= out_ACtr;
		else
			out_mux_last <= x"0000000000000000000000000000000000000000000000000000000000000000";
		end if;
	end process;
	
	
	
	
	inst_SC: swapcolumns
		port map(in_SC(255 downto 128), in_SC(127 downto 0), out_SC(255 downto 128), out_SC(127 downto 0));
		
	ark <= out_SC;
		
   counter_integer <= conv_integer(counter);
	section_number_integer <= conv_integer(section_number);
	roundnumber <= conv_std_logic_vector(8*(counter_integer + section_number_integer*section_multiplyer + plusval), 9);
	ciphertext <= ark_reg;
	
	--hier moeten we nu ook een keuze maken!
	p_roundnumber: process(clk)
	begin
		if clk'event and clk='1' then
			case rounds is
			   when '1' =>
			      section_multiplyer <= 5;
			      plusval <= 0;
			   when others =>
			      section_multiplyer <= 2;
			      plusval <= 30;			
			end case;
		end if;
	end process;

	p_rcon: process(counter)
	begin
		if rounds = '1'then
			case counter is
				when "101" =>
					last <= '1';
				when others =>
					last <= '0';
			end case;
		else 
			case counter is
				when "010" =>
					last <= '1';
				when others =>
					last <= '0';
			end case;		
		end if;
	end process;


	fsm: process(rst, clk)
	begin
		if rst = '0' then
			load2 <= '0';
			en <= '0';
			counter <= "000";
			done <= '0';
			state <= idle;
		elsif clk'event and clk = '1' then
			case state is
				when idle =>
					if start = '1' then
						load2 <= '1';
						en <= '1';
						counter <= "000";
						done <= '0';
						state <= state1;
					else
						load2 <= '0';
						en <= '0';
						counter <= "000";
						done <= '0';
						state <= idle;
					end if;
				when state1 =>
					load2 <= '0';
					en <= '1';
					counter <= counter + 1;
					done <= '0';
					state <= state2;
				when state2 =>
					if last = '1' then
						load2 <= '0';
						en <= '0';
						counter <= "000";
						done <= '1';
						state <= idle;
					else
						load2 <= '0';
						en <= '1';
						counter <= counter + 1;
						done <= '0';
						state <= state2;
					end if;
				when others =>
					load2 <= '0';
					en <= '0';
					counter <= "000";
					done <= '0';
					state <= idle;
			end case;
		end if;
	end process;
	
end Behavioral;