
---------------------------------------
--
-- Testbench voor LANE_full
--
-- Masterproef 
--
---------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TB_Lane_full is
end TB_Lane_full;

architecture behav of TB_Lane_full is
    
	component Lane_full is port(
		clk, rst, init, add, final: in std_logic;
		done: out std_logic;
		data_in: in std_logic_vector(511 downto 0);
		data_bits: in std_logic_vector(9 downto 0);
		hash_out: out std_logic_vector(255 downto 0));
    end component;
 
	--signalen
	SIGNAL data_in : STD_LOGIC_VECTOR(511 downto 0);   
	SIGNAL data_bits: std_logic_vector(9 downto 0);    
	SIGNAL clk :  std_logic := '0';
	SIGNAL rst :  std_logic := '0';
	SIGNAL init :  std_logic := '0';
	SIGNAL add :  std_logic := '0';
	SIGNAL final :  std_logic := '0';
	SIGNAL done :  std_logic;
	SIGNAL hash_out: STD_LOGIC_VECTOR(255 downto 0);
    
	--signalen en constanten voor simulatiedoeleinden
	signal EndOfSim   : boolean:=false;
	constant ClkPeriod : time := 10 ns;
	constant startlevel: std_logic := '0';
  
begin
    
	inst_Lane: Lane_full
    port map(clk, rst, init, add, final,done,data_in,data_bits,hash_out);
    
	
	p_clk: process
	begin
		loop
        exit when EndOfSim;
        clk <= startlevel;
        wait for CLKPeriod / 2;
        exit when EndOfSim;
        clk <= not startlevel;
        wait for CLKPeriod - (CLKPeriod / 2); 
      end loop;
	end process;
	
	
	testprocess: process
	begin
					EndOfSim <= false;
					wait for 10ns;
					data_in <= x"6162636465666768696a6b6c6d6e6f707172737475767778797a4142434445464748494a4b4c4d4e4f505152535455565758595a303132333435363738396162";
					data_bits <= "1000000000"; --512
					rst<='1'; --laag actieve reset
					
					--init compressionfunction
					init<='1';
					report"*-*-*-*-*-*-*-*-*-*Start simulatie*-*-*-*-*-*-*-*-*";
					wait until done = '1';
					init<='0';
					report"------init done";
					
					--hash block1
					add<='1';
					wait for 15ns;
					add<='0';
					wait until done = '1';
					
					report"------add done";
					Assert( hash_out = x"9DB9E5471C1EDF25C66DF17616015F1EFD8914CA48F3F2333BB7AFA4DF87F1D1" ) report "De hashwaarde is fout!" severity ERROR;
					
					data_in <= x"636465666768696a6b6c6d6e6f707172737475767778797a4142434445464748494a4b4c4d4e4f505152535455565758595a3031323334353637383900000000";
					data_bits <= "0111100000"; --480
					--hash block 2
					add<='1';
					wait for 15ns;
					add<='0';
					wait until done = '1';
					report"------add done";
					Assert( hash_out = x"1CB38AAC0B86B44B1D66C016BB88EFA5DA30A7533BD014AF7B9282DDB74C7059" ) report "De hashwaarde is fout!" severity ERROR;
					
					--finalize hash
					wait for 15ns;
					final<='1';
					wait for 15ns;
					final<='0';
					wait until done = '1';
					
					
					report"------final done";
					Assert( hash_out = x"BBB8676FCB170EC57046DD1EFCA59270E00E68D818D4C23235CDCE873604F1BB" ) report "De hashwaarde is fout!" severity ERROR;
					EndOfSim <= true;
					report"*-*-*-*-*-*-*-*-*-*Einde simulatie*-*-*-*-*-*-*-*-*";
	  
    end process;
    
end behav;

   