
---------------------------------------
--
-- Testbench voor LANE hashfunctie
--
-- Masterproef 
--
---------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TB_round is
end TB_round;

architecture behav of TB_round is
    
 component round is
    Port ( plaintext : in  STD_LOGIC_VECTOR (255 downto 0);
           ciphertext : out  STD_LOGIC_VECTOR (255 downto 0);
           rst, clk : in STD_LOGIC;
			  start : in  STD_LOGIC;
           done : out  STD_LOGIC;
			  bit_counter : in STD_LOGIC_VECTOR(63 downto 0);
			  section_number : in STD_LOGIC_VECTOR(2 downto 0);
			  rounds : in  STD_LOGIC
			  );
end component;
 
	--signalen
	SIGNAL plain : STD_LOGIC_VECTOR(255 downto 0);
	SIGNAL cipher : STD_LOGIC_VECTOR(255 downto 0);       
	SIGNAL clk :  std_logic := '0';
	SIGNAL rst :  std_logic := '0';
	SIGNAL start :  std_logic := '0';
	SIGNAL done :  std_logic;
	SIGNAL bit_counter : STD_LOGIC_VECTOR(63 downto 0);
	SIGNAL section: STD_LOGIC_VECTOR(2 downto 0);
	signal rounds:  STD_LOGIC;
    
	--signalen en constanten voor simulatiedoeleinden
	signal EndOfSim   : boolean:=false;
	constant ClkPeriod : time := 10 ns;
	constant startlevel: std_logic := '0';
  
begin
    
	inst_round: round
    port map(plain,cipher,rst, clk ,start,done,bit_counter,section,rounds);
    
	
	p_clk: process
	begin
		loop
        exit when EndOfSim;
        clk <= startlevel;
        wait for CLKPeriod / 2;
        exit when EndOfSim;
        clk <= not startlevel;
        wait for CLKPeriod - (CLKPeriod / 2); 
      end loop;
	end process;
	
	
	
	testprocess: process
	begin
					EndOfSim <= false;
					
					--eerst een P-permutatie testen
					wait for 10ns;
					bit_counter <=    "0000000000000000000000000000000000000000000000000000000000000000"; --zet de bitcounter op 0
					section <= "000";
					rounds <= '1';
					--eerste P0 uit intermediate file
					plain<= 	     x"0200000100000000000000000000000002000001000000000000000000000000"; --input
					rst<='1'; --laag actieve reset
					start<='1';
					report"*-*-*-*-*-*-*-*-*-*Start simulatie*-*-*-*-*-*-*-*-*";
					wait for 10ns;
					start<='0';
					wait until done = '1';
					wait for 1ns;
					--nu kijken of op de uitgang staat wat er moet staan
					Assert( cipher = x"F4F83ED375DB6BE557F213D00A5959F0398BFF632AC4AC04F2ADF3736F304F2C" ) report "De P-permutatiewaarde gaat fout!" severity ERROR;
					
					--dan ook nog een Q-permutatie testen
					wait for 10ns;
					bit_counter <=    "0000000000000000000000000000000000000000000000000000000111110000"; --zet de bitcounter op 0
					section <= "000";
					rounds <= '0';
					--eerste P0 uit intermediate file
					plain<= 	     x"AA52D00EC0F3C336EBB190B1B12AAC6354752F073EB907FF25F66125C990F007"; --input
					rst<='1'; --laag actieve reset
					start<='1';
					wait for 10ns;
					start<='0';
					wait until done = '1';
					wait for 1ns;
					--nu kijken of op de uitgang staat wat er moet staan
					Assert( cipher = x"E3819EE587FCBC2E0AE3AB88F8FF37D52810BB7255F52F990D0FE5C86DFE924A" ) report "De Q-permutatiewaarde gaat fout!" severity ERROR;
					
					
					
					
					report"*-*-*-*-*-*-*-*-*-*Einde simulatie*-*-*-*-*-*-*-*-*";
					EndOfSim <= true;
	  
    end process;
    
end behav;

   