-- in deze blok zitten 2 counters(een voor aantal boodschappen,een voor aantal 
--binnengestroomde bits), message padding, een 32bitBuffer
-- 
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Luffa_Buffer is
	port (	input,init,end_last_message, clk, rst,enable: in std_logic;
			 output: out std_logic_vector(255 downto 0);
			 done,data_ready,init_ready: out std_logic);
end Luffa_Buffer;

architecture Buffer_arch of Luffa_Buffer is


component FFBuffer is
	port (	buffer_in, clk, rst,enable_buffer: in std_logic;
			 buffer_out: out std_logic_vector(255 downto 0));
end component;

component Counter9bit is
    port(enable_9bc, clk, rst: in std_logic;
         enable_buffer: out std_logic;
         output : out std_logic_vector(8 downto 0)
        );
end component;
   
component MessagePadding is
    port(enable_MP, clk, rst,preset: in std_logic;
         output_MP: out std_logic
        );
end component;
   

component enable_FSM is
	port (	Enable_in,clk,rst: in std_logic;
			 enable_out: out std_logic);
end component;

component Data_ready_fsm is
	port (	data_ready_in,clk,rst: in std_logic;
			 Data_ready: out std_logic);
end component;

type my_state is (w0,W1,done_state,waiting,idle,init_State,running,messagepadding_State,finalization);

signal state :my_state;
signal input_buffer,input_mp,data_ready_length: std_logic; 
signal enable_buffer,enable_bufferinput,enable_out,enable_9bitcounter,enable_mp: std_logic;
signal preset: std_logic; 
signal rst_counter,rst_9bc,data_ready_9bc,DR,init_ready_int,finalize,rst_buffer,data_ready_int: std_logic;
signal output_reg,output_buffer:std_logic_vector(255 downto 0);

signal BC_Out : std_logic_Vector(8 downto 0);

begin
inst_Data_ready_fsm_9bc: Data_ready_fsm
	port map (	data_ready_int,clk,rst,DR);
data_ready_int <= data_ready_9bc xor finalize;
data_ready <= DR ;

output <= output_reg; --kan evt weggelaten worden maar dan moete timing tussen 2 blokken perfect zijn... 
--alhoewel: data ready is gewoon load in MI_P dus die hoeft mr half in fsm opgenome te worden 
--(zelfs dat niet als we extra reset op buffer zetten met nog een data_ready zodat "0000...000" doorgegeven wordt als finalization:-)



inst_256bitBuffer: FFBuffer
      port map(input_buffer,clk,rst_buffer,enable_buffer,output_buffer);
 
inst_messagepadding: MessagePadding
      port map(enable_mp,clk,rst,preset,input_mp);

inst_9bitcounter: Counter9bit
      port map(enable_9bitcounter,clk,rst_9bc,enable_bufferinput,BC_out);
          
inst_enable_FSM: enable_FSM
      port map(enable,clk,rst,enable_out);

rst_buffer <= rst or finalize;
       
data_ready_9bc  <= not(enable_bufferinput);
enable_buffer <= enable_out and enable_bufferinput;
enable_9bitcounter <= enable_out and enable_bufferinput;
enable_mp <= end_last_message and enable_buffer;
rst_9bc <= rst or rst_counter;

-- in het volgende deel wordt bepaald wat er aan de ingang van de 256bitBuffer en output_Reg wordt gehangen.
-- nl de normale input van de powerpc of de blok MessagePadding of resetten nr 000000000... voor finalisatie
process(finalize,rst,clk)
begin
	
	if finalize = '1' or rst = '1' then
		output_reg <= (others => '0');
	elsif rising_edge(clk) then
		if data_ready_9bc = '1' then
		  output_reg <= output_buffer;
				
		else output_reg <= output_reg;
		end if;
	end if;
end process;
		  
process(end_last_message,input_mp,input)
    begin
        if end_last_message = '1' then
            input_buffer <= input_mp;
        else
            input_buffer <= input;
    end if;
end process;




init_ready <= init_Ready_int;	    	
fsm: process(rst,clk)--data_ready_length verwijderd
begin 
   	if rst = '1' then
			  preset <= '0';
			  init_ready_int <= '0';		
			  done <= '0';			 
			  state <= idle;
			  rst_counter <= '0';
			  
			  finalize <= '0';
	 elsif rising_edge(clk) then   
			case state is
				when idle =>	
					if init = '1' AND init_ready_int = '0' then
				      preset <= '1';
				      init_ready_int <= '0';
				      done <= '0';						
						state <= init_State;
						rst_counter <= '0';						
						
						finalize <= '0';
					else--if init_ready_int ='1' then
						preset <= '1'; -- asynchrone preset
						init_ready_int<='0';
						done <= '0';
						state <= running;
						rst_counter <= '0';						
						
						finalize <= '0';
					end if;
				when init_State =>
					   preset <= '0';
					   init_ready_int <= '1';
					   done <= '0';
					   state <= running;
						rst_counter <= '0';
						
						finalize <= '0';
				when running =>
					if end_last_message = '1' then
					   preset <= '0'; 
					   init_ready_int <= '1';
					   done <= '0';
					   state <= messagepadding_State; --messagepadding_State;
						rst_counter <= '0';
						
						finalize <= '0';
					elsif data_ready_9bc='1' then
					   preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
				      state <= running;
						rst_counter <= '1';
						
						finalize <= '0';
					else 
					   preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
				      state <= running;
						rst_counter <= '0';
						
						finalize <= '0';
					end if;
					
				
				      
				
				
				
					
				when messagepadding_State =>
				   if data_ready_9bc= '1' then 
				      preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
				      state <=finalization;
						rst_counter <= '1';
						
						finalize <= '0';
				   else
				      preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
				      state <= messagepadding_state;
						rst_counter <= '0';
						
						finalize <= '0';
				   end if;
				when waiting =>
						preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						
						state <= finalization;
						rst_counter <= '1';
						finalize <= '0';
				when finalization =>
						preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						
						state <= done_state;
						rst_counter <= '1';
						finalize <= '1';
				when done_state =>
						preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '1';
						
						state <= done_state;
						rst_counter <= '1';
						finalize <= '0';
				when others =>
				      preset <= '0'; 
				      init_ready_int <= '1';
				      done <= '0';
						
						state <= idle;
						rst_counter <= '0';
						finalize <= '0';
			end case;
		end if;
	end process;     


	
end Buffer_arch;
	
	
